@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,9 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - contributions for 
+ *     							bug 337868 - [compiler][model] incomplete support for package-info.java when using SearchableEnvironment
+ *     							bug 186342 - [compiler][null] Using annotations for null checking
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler;
 
@@ -352,6 +355,9 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 		ICompilationUnit sourceUnit,
 		CompilationUnitDeclaration parsedUnit) {
 
+		if (this.unitsToProcess == null)
+			return; // not collecting units
+
 		// append the unit to the list of ones to process later on
 		int size = this.unitsToProcess.length;
 		if (this.totalUnits == size)
@@ -586,17 +592,17 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 			result
 				.record(
 					this.problemReporter
-					.createProblem(
-						result.getFileName(),
-						IProblem.Unclassified,
-						pbArguments,
-						pbArguments,
-						Error, // severity
-						0, // source start
-						0, // source end
-						0, // line number
-						0),// column number
-					unit);
+						.createProblem(
+							result.getFileName(),
+							IProblem.Unclassified,
+							pbArguments,
+							pbArguments,
+							Error, // severity
+							0, // source start
+							0, // source end
+							0, // line number
+							0),// column number
+					unit, true);
 
 			/* hand back the compilation result */
 			if (!result.hasBeenAccepted) {
@@ -658,7 +664,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 					if (distantProblem instanceof DefaultProblem) { // fixup filename TODO (philippe) should improve API to make this official
 						((DefaultProblem) distantProblem).setOriginatingFileName(result.getFileName());
 					}
-					result.record(distantProblem, unit);
+					result.record(distantProblem, unit, true);
 				}
 			} else {
 				/* distant internal exception which could not be reported back there */
@@ -691,6 +697,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 
 		// Switch the current policy and compilation result for this unit to the requested one.
 		for (int i = 0; i < maxUnits; i++) {
+			CompilationResult unitResult = null;
 			try {
 				if (this.options.verbose) {
 					this.out.println(
@@ -703,8 +710,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 				}
 				// diet parsing for large collection of units
 				CompilationUnitDeclaration parsedUnit;
-				CompilationResult unitResult =
-					new CompilationResult(sourceUnits[i], i, maxUnits, this.options.maxProblemsPerUnit);
+				unitResult = new CompilationResult(sourceUnits[i], i, maxUnits, this.options.maxProblemsPerUnit);
 				long parseStart = System.currentTimeMillis();
 				if (this.totalUnits < this.parseThreshold) {
 					parsedUnit = this.parser.parse(sourceUnits[i], unitResult);
@@ -723,6 +729,11 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 				}
 				//} catch (AbortCompilationUnit e) {
 				//	requestor.acceptResult(unitResult.tagAsAccepted());
+			} catch (AbortCompilation a) {
+				// best effort to find a way for reporting this problem:
+				if (a.compilationResult == null)
+					a.compilationResult = unitResult;
+				throw a;
 			} finally {
 				sourceUnits[i] = null; // no longer hold onto the unit
 			}
@@ -760,16 +771,15 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 
 		long analyzeStart = System.currentTimeMillis();
 		this.stats.resolveTime += analyzeStart - resolveStart;
-
-		// flow analysis
-		unit.analyseCode();
+		
+		//No need of analysis or generation of code if statements are not required		
+		if (!this.options.ignoreMethodBodies) unit.analyseCode(); // flow analysis
 
 		long generateStart = System.currentTimeMillis();
 		this.stats.analyzeTime += generateStart - analyzeStart;
-
-		// code generation
-		unit.generateCode();
-
+	
+		if (!this.options.ignoreMethodBodies) unit.generateCode(); // code generation
+		
 		// reference info
 		if (this.options.produceReferenceInfo && unit.scope != null)
 			unit.scope.storeDependencyInfo();
@@ -832,10 +842,26 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 				this.annotationProcessorManager.reset();
 			}
 		} while (newUnitSize != 0 || newClassFilesSize != 0);
-		// one more loop to create possible resources
-		// this loop cannot create any java source files
+		
 		this.annotationProcessorManager.processAnnotations(null, null, true);
-		// TODO we might want to check if this loop created new units
+		// process potential units added in the final round see 329156 
+		ICompilationUnit[] newUnits = this.annotationProcessorManager.getNewUnits();
+		newUnitSize = newUnits.length;
+		if (newUnitSize != 0) {
+			ICompilationUnit[] newProcessedUnits = (ICompilationUnit[]) newUnits.clone(); // remember new units in case a source type collision occurs
+			try {
+				this.lookupEnvironment.isProcessingAnnotations = true;
+				internalBeginToCompile(newUnits, newUnitSize);
+			} catch (SourceTypeCollisionException e) {
+				e.newAnnotationProcessorUnits = newProcessedUnits;
+				throw e;
+			} finally {
+				this.lookupEnvironment.isProcessingAnnotations = false;
+				this.annotationProcessorManager.reset();
+			}
+		} else {
+			this.annotationProcessorManager.reset();
+		}
 	}
 
 	public void reset() {
@@ -861,8 +887,18 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 				// build and record parsed units
 				this.parseThreshold = 0; // will request a full parse
 				beginToCompile(new ICompilationUnit[] { sourceUnit });
-				// process all units (some more could be injected in the loop by the lookup environment)
-				unit = this.unitsToProcess[0];
+				// find the right unit from what was injected via accept(ICompilationUnit,..):
+				for (int i=0; i<this.totalUnits; i++) {
+					if (   this.unitsToProcess[i] != null
+						&& this.unitsToProcess[i].compilationResult.compilationUnit == sourceUnit)
+					{
+						unit = this.unitsToProcess[i];
+						break;
+					}
+				}
+				if (unit == null)
+					unit = this.unitsToProcess[0]; // fall back to old behavior
+
 			} else {
 				// initial type binding creation
 				this.lookupEnvironment.buildTypeBindings(unit, null /*no access restriction*/);
