@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2008 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,9 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contribution for
+ *								bug 345305 - [compiler][null] Compiler misidentifies a case of "variable can only be null"
+ *								bug 383368 - [compiler][null] syntactic null analysis for field references
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -18,9 +21,9 @@ import org.eclipse.jdt.internal.compiler.flow.*;
 import org.eclipse.jdt.internal.compiler.lookup.*;
 
 public class BinaryExpression extends OperatorExpression {
-	
+
 /* Tracking helpers
- * The following are used to elaborate realistic statistics about binary 
+ * The following are used to elaborate realistic statistics about binary
  * expressions. This must be neutralized in the released code.
  * Search the keyword BE_INSTRUMENTATION to reenable.
  * An external device must install a suitable probe so as to monitor the
@@ -42,7 +45,7 @@ public BinaryExpression(Expression left, Expression right, int operator) {
 	this.sourceStart = left.sourceStart;
 	this.sourceEnd = right.sourceEnd;
 	// BE_INSTRUMENTATION: neutralized in the released code
-//	if (left instanceof BinaryExpression && 
+//	if (left instanceof BinaryExpression &&
 //			((left.bits & OperatorMASK) ^ (this.bits & OperatorMASK)) == 0) {
 //		this.depthTracker = ((BinaryExpression)left).depthTracker + 1;
 //	} else {
@@ -56,19 +59,30 @@ public BinaryExpression(BinaryExpression expression) {
 	this.sourceStart = expression.sourceStart;
 	this.sourceEnd = expression.sourceEnd;
 }
-public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, 
-		FlowInfo flowInfo) {
+public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 	// keep implementation in sync with CombinedBinaryExpression#analyseCode
-	if (this.resolvedType.id == TypeIds.T_JavaLangString) {
-		return this.right.analyseCode(
-							currentScope, flowContext, 
-							this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits())
-						.unconditionalInits();
-	} else {
-		this.left.checkNPE(currentScope, flowContext, flowInfo);
-		flowInfo = this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
-		this.right.checkNPE(currentScope, flowContext, flowInfo);
-		return this.right.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
+	try {
+		if (this.resolvedType.id == TypeIds.T_JavaLangString) {
+			return this.right.analyseCode(
+								currentScope, flowContext,
+								this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits())
+							.unconditionalInits();
+		} else {
+			this.left.checkNPE(currentScope, flowContext, flowInfo);
+			flowInfo = this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
+			if (((this.bits & OperatorMASK) >> OperatorSHIFT) != AND) {
+				flowContext.expireNullCheckedFieldInfo();
+			}
+			this.right.checkNPE(currentScope, flowContext, flowInfo);
+			flowInfo = this.right.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
+			if (((this.bits & OperatorMASK) >> OperatorSHIFT) != AND) {
+				flowContext.expireNullCheckedFieldInfo();
+			}
+			return flowInfo;
+		}
+	} finally {
+		// account for exception possibly thrown by arithmetics
+		flowContext.recordAbruptExit();
 	}
 }
 
@@ -91,7 +105,7 @@ public void computeConstant(BlockScope scope, int leftId, int rightId) {
 		}
 	} else {
 		this.constant = Constant.NotAConstant;
-		//add some work for the boolean operators & |  
+		//add some work for the boolean operators & |
 		this.optimizedBooleanConstant(
 			leftId,
 			(this.bits & ASTNode.OperatorMASK) >> ASTNode.OperatorSHIFT,
@@ -121,7 +135,7 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 		case PLUS :
 			switch (this.bits & ASTNode.ReturnTypeIDMASK) {
 				case T_JavaLangString :
-					// BE_INSTRUMENTATION: neutralized in the released code					
+					// BE_INSTRUMENTATION: neutralized in the released code
 //					if (probe != null) {
 //						probe.ping(this.depthTracker);
 //					}
@@ -670,7 +684,7 @@ public void generateOptimizedGreaterThan(BlockScope currentScope, CodeStream cod
 				}
 			}
 			// reposition the endPC
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 		// x > 0
@@ -692,7 +706,7 @@ public void generateOptimizedGreaterThan(BlockScope currentScope, CodeStream cod
 				}
 			}
 			// reposition the endPC
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 	}
@@ -720,7 +734,7 @@ public void generateOptimizedGreaterThan(BlockScope currentScope, CodeStream cod
 						codeStream.ifgt(trueLabel);
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			}
 		} else {
@@ -743,7 +757,7 @@ public void generateOptimizedGreaterThan(BlockScope currentScope, CodeStream cod
 						codeStream.ifle(falseLabel);
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			} else {
 				// no implicit fall through TRUE/FALSE --> should never occur
@@ -778,7 +792,7 @@ public void generateOptimizedGreaterThanOrEqual(BlockScope currentScope, CodeStr
 				}
 			}
 			// reposition the endPC
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 		// x >= 0
@@ -800,7 +814,7 @@ public void generateOptimizedGreaterThanOrEqual(BlockScope currentScope, CodeStr
 				}
 			}
 			// reposition the endPC
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 	}
@@ -828,7 +842,7 @@ public void generateOptimizedGreaterThanOrEqual(BlockScope currentScope, CodeStr
 						codeStream.ifge(trueLabel);
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			}
 		} else {
@@ -851,7 +865,7 @@ public void generateOptimizedGreaterThanOrEqual(BlockScope currentScope, CodeStr
 						codeStream.iflt(falseLabel);
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			} else {
 				// no implicit fall through TRUE/FALSE --> should never occur
@@ -885,7 +899,7 @@ public void generateOptimizedLessThan(BlockScope currentScope, CodeStream codeSt
 					}
 				}
 			}
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 		// x < 0
@@ -906,7 +920,7 @@ public void generateOptimizedLessThan(BlockScope currentScope, CodeStream codeSt
 					}
 				}
 			}
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 	}
@@ -933,7 +947,7 @@ public void generateOptimizedLessThan(BlockScope currentScope, CodeStream codeSt
 						codeStream.dcmpg();
 						codeStream.iflt(trueLabel);
 				}
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			}
 		} else {
@@ -955,7 +969,7 @@ public void generateOptimizedLessThan(BlockScope currentScope, CodeStream codeSt
 						codeStream.dcmpg();
 						codeStream.ifge(falseLabel);
 				}
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			} else {
 				// no implicit fall through TRUE/FALSE --> should never occur
@@ -990,7 +1004,7 @@ public void generateOptimizedLessThanOrEqual(BlockScope currentScope, CodeStream
 				}
 			}
 			// reposition the endPC
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 		// x <= 0
@@ -1012,7 +1026,7 @@ public void generateOptimizedLessThanOrEqual(BlockScope currentScope, CodeStream
 				}
 			}
 			// reposition the endPC
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			return;
 		}
 	}
@@ -1040,7 +1054,7 @@ public void generateOptimizedLessThanOrEqual(BlockScope currentScope, CodeStream
 						codeStream.ifle(trueLabel);
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			}
 		} else {
@@ -1063,7 +1077,7 @@ public void generateOptimizedLessThanOrEqual(BlockScope currentScope, CodeStream
 						codeStream.ifgt(falseLabel);
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				return;
 			} else {
 				// no implicit fall through TRUE/FALSE --> should never occur
@@ -1091,10 +1105,10 @@ public void generateLogicalAnd(BlockScope currentScope, CodeStream codeStream, b
 					codeStream.iconst_0();
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			}
 			return;
-		} 
+		}
 		if ((condConst = this.right.optimizedBooleanConstant()) != Constant.NotAConstant) {
 			if (condConst.booleanValue() == true) {
 				// x & <something equivalent to true>
@@ -1108,7 +1122,7 @@ public void generateLogicalAnd(BlockScope currentScope, CodeStream codeStream, b
 					codeStream.iconst_0();
 				}
 				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			}
 			return;
 		}
@@ -1119,8 +1133,7 @@ public void generateLogicalAnd(BlockScope currentScope, CodeStream codeStream, b
 	if (valueRequired) {
 		codeStream.iand();
 	}
-	// reposition the endPC
-	codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 }
 
 /**
@@ -1137,8 +1150,7 @@ public void generateLogicalOr(BlockScope currentScope, CodeStream codeStream, bo
 				if (valueRequired) {
 					codeStream.iconst_1();
 				}
-				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			} else {
 				// <something equivalent to false> | x
 				this.left.generateCode(currentScope, codeStream, false);
@@ -1154,8 +1166,7 @@ public void generateLogicalOr(BlockScope currentScope, CodeStream codeStream, bo
 				if (valueRequired) {
 					codeStream.iconst_1();
 				}
-				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			} else {
 				// x | <something equivalent to false>
 				this.left.generateCode(currentScope, codeStream, valueRequired);
@@ -1170,8 +1181,7 @@ public void generateLogicalOr(BlockScope currentScope, CodeStream codeStream, bo
 	if (valueRequired) {
 		codeStream.ior();
 	}
-	// reposition the endPC
-	codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 }
 
 /**
@@ -1190,7 +1200,7 @@ public void generateLogicalXor(BlockScope currentScope,	CodeStream codeStream, b
 				this.right.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					codeStream.ixor(); // negate
-					codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+					codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				}
 			} else {
 				// <something equivalent to false> ^ x
@@ -1207,7 +1217,7 @@ public void generateLogicalXor(BlockScope currentScope,	CodeStream codeStream, b
 				if (valueRequired) {
 					codeStream.iconst_1();
 					codeStream.ixor(); // negate
-					codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+					codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 				}
 			} else {
 				// x ^ <something equivalent to false>
@@ -1223,9 +1233,8 @@ public void generateLogicalXor(BlockScope currentScope,	CodeStream codeStream, b
 	if (valueRequired) {
 		codeStream.ixor();
 	}
-	// reposition the endPC
-	codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
-}	
+	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
+}
 
 /**
  * Boolean generation for &
@@ -1268,8 +1277,7 @@ public void generateOptimizedLogicalAnd(BlockScope currentScope, CodeStream code
 						codeStream.goto_(falseLabel);
 					}
 				}
-				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			}
 			return;
 		}
@@ -1310,8 +1318,7 @@ public void generateOptimizedLogicalAnd(BlockScope currentScope, CodeStream code
 						codeStream.goto_(falseLabel);
 					}
 				}
-				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			}
 			return;
 		}
@@ -1335,8 +1342,7 @@ public void generateOptimizedLogicalAnd(BlockScope currentScope, CodeStream code
 			}
 		}
 	}
-	// reposition the endPC
-	codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 }
 
 /**
@@ -1367,8 +1373,7 @@ public void generateOptimizedLogicalOr(BlockScope currentScope, CodeStream codeS
 						codeStream.goto_(trueLabel);
 					}
 				}
-				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			} else {
 				// <something equivalent to false> | x
 				this.left.generateOptimizedBoolean(
@@ -1408,8 +1413,7 @@ public void generateOptimizedLogicalOr(BlockScope currentScope, CodeStream codeS
 						codeStream.goto_(trueLabel);
 					}
 				}
-				// reposition the endPC
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+				codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 			} else {
 				// x | <something equivalent to false>
 				this.left.generateOptimizedBoolean(
@@ -1447,8 +1451,7 @@ public void generateOptimizedLogicalOr(BlockScope currentScope, CodeStream codeS
 			}
 		}
 	}
-	// reposition the endPC
-	codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 }
 
 /**
@@ -1541,8 +1544,7 @@ public void generateOptimizedLogicalXor(BlockScope currentScope, CodeStream code
 			}
 		}
 	}
-	// reposition the endPC
-	codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 }
 
 public void generateOptimizedStringConcatenation(BlockScope blockScope, CodeStream codeStream, int typeID) {
@@ -1581,7 +1583,7 @@ public void generateOptimizedStringConcatenationCreation(BlockScope blockScope,
 	// keep implementation in sync with CombinedBinaryExpression
 	// #generateOptimizedStringConcatenationCreation
 	/* In the case trying to make a string concatenation, there is no need to create a new
-	 * string buffer, thus use a lower-level API for code generation involving only the 
+	 * string buffer, thus use a lower-level API for code generation involving only the
 	 * appending of arguments to the existing StringBuffer
 	 */
 	if ((((this.bits & ASTNode.OperatorMASK) >> ASTNode.OperatorSHIFT) == OperatorIds.PLUS)
@@ -1616,9 +1618,9 @@ public boolean isCompactableOperation() {
 }
 
 /**
- * Separates into a reusable method the subpart of {@link 
- * #resolveType(BlockScope)} that needs to be executed while climbing up the 
- * chain of expressions of this' leftmost branch. For use by {@link 
+ * Separates into a reusable method the subpart of {@link
+ * #resolveType(BlockScope)} that needs to be executed while climbing up the
+ * chain of expressions of this' leftmost branch. For use by {@link
  * CombinedBinaryExpression#resolveType(BlockScope)}.
  * @param scope the scope within which the resolution occurs
  */
@@ -1626,7 +1628,7 @@ void nonRecursiveResolveTypeUpwards(BlockScope scope) {
 	// keep implementation in sync with BinaryExpression#resolveType
 	boolean leftIsCast, rightIsCast;
 	TypeBinding leftType = this.left.resolvedType;
-	
+
 	if ((rightIsCast = this.right instanceof CastExpression) == true) {
 		this.right.bits |= ASTNode.DisableUnnecessaryCastCheck; // will check later on
 	}
@@ -1684,7 +1686,7 @@ void nonRecursiveResolveTypeUpwards(BlockScope scope) {
 	//  <<16   <<12       <<8    <<4       <<0
 
 	// Don't test for result = 0. If it is zero, some more work is done.
-	// On the one hand when it is not zero (correct code) we avoid doing the test	
+	// On the one hand when it is not zero (correct code) we avoid doing the test
 	int operator = (this.bits & ASTNode.OperatorMASK) >> ASTNode.OperatorSHIFT;
 	int operatorSignature = OperatorExpression.OperatorSignatures[operator][(leftTypeID << 4) + rightTypeID];
 
@@ -1724,7 +1726,7 @@ void nonRecursiveResolveTypeUpwards(BlockScope scope) {
 	}
 
 	// check need for operand cast
-	if ((leftIsCast = (this.left instanceof CastExpression)) == true || 
+	if ((leftIsCast = (this.left instanceof CastExpression)) == true ||
 			rightIsCast) {
 		CastExpression.checkNeedForArgumentCasts(scope, operator, operatorSignature, this.left, leftTypeID, leftIsCast, this.right, rightTypeID, rightIsCast);
 	}
@@ -1737,6 +1739,7 @@ public void optimizedBooleanConstant(int leftId, int operator, int rightId) {
 		case AND :
 			if ((leftId != TypeIds.T_boolean) || (rightId != TypeIds.T_boolean))
 				return;
+			//$FALL-THROUGH$
 		case AND_AND :
 			Constant cst;
 			if ((cst = this.left.optimizedBooleanConstant()) != Constant.NotAConstant) {
@@ -1760,6 +1763,7 @@ public void optimizedBooleanConstant(int leftId, int operator, int rightId) {
 		case OR :
 			if ((leftId != TypeIds.T_boolean) || (rightId != TypeIds.T_boolean))
 				return;
+			//$FALL-THROUGH$
 		case OR_OR :
 			if ((cst = this.left.optimizedBooleanConstant()) != Constant.NotAConstant) {
 				if (cst.booleanValue() == true) { // left is equivalent to true
@@ -1781,7 +1785,7 @@ public void optimizedBooleanConstant(int leftId, int operator, int rightId) {
 }
 
 public StringBuffer printExpressionNoParenthesis(int indent, StringBuffer output) {
-	// keep implementation in sync with 
+	// keep implementation in sync with
 	// CombinedBinaryExpression#printExpressionNoParenthesis
 	this.left.printExpression(indent, output).append(' ').append(operatorToString()).append(' ');
 	return this.right.printExpression(0, output);
@@ -1849,11 +1853,11 @@ public TypeBinding resolveType(BlockScope scope) {
 	//  <<16   <<12       <<8    <<4       <<0
 
 	// Don't test for result = 0. If it is zero, some more work is done.
-	// On the one hand when it is not zero (correct code) we avoid doing the test	
+	// On the one hand when it is not zero (correct code) we avoid doing the test
 	int operator = (this.bits & ASTNode.OperatorMASK) >> ASTNode.OperatorSHIFT;
 	int operatorSignature = OperatorExpression.OperatorSignatures[operator][(leftTypeID << 4) + rightTypeID];
 
-	this.left.computeConversion(scope, 	TypeBinding.wellKnownType(scope, (operatorSignature >>> 16) & 0x0000F), leftType);
+	this.left.computeConversion(scope, TypeBinding.wellKnownType(scope, (operatorSignature >>> 16) & 0x0000F), leftType);
 	this.right.computeConversion(scope, TypeBinding.wellKnownType(scope, (operatorSignature >>> 8) & 0x0000F), rightType);
 	this.bits |= operatorSignature & 0xF;
 	switch (operatorSignature & 0xF) { // record the current ReturnTypeID
