@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2008 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,9 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Patrick Wienands <pwienands@abit.de> - Contribution for bug 393749
+ *     Stephan Herrmann - Contribution for
+ *								bug 331649 - [compiler][null] consider null annotations for fields
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -16,28 +19,30 @@ import org.eclipse.jdt.internal.compiler.CompilationResult;
 import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
 import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
 import org.eclipse.jdt.internal.compiler.codegen.ConstantPool;
+import org.eclipse.jdt.internal.compiler.codegen.Opcodes;
 import org.eclipse.jdt.internal.compiler.flow.ExceptionHandlingFlowContext;
 import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
 import org.eclipse.jdt.internal.compiler.flow.InitializationFlowContext;
 import org.eclipse.jdt.internal.compiler.lookup.Binding;
-import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
 import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
 import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
 import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
 import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
+import org.eclipse.jdt.internal.compiler.lookup.SyntheticMethodBinding;
 import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.eclipse.jdt.internal.compiler.parser.Parser;
 import org.eclipse.jdt.internal.compiler.problem.AbortMethod;
 
 public class Clinit extends AbstractMethodDeclaration {
-	
+	private static int ENUM_CONSTANTS_THRESHOLD = 2000;
+
 	private FieldBinding assertionSyntheticFieldBinding = null;
 	private FieldBinding classLiteralSyntheticField = null;
 
 	public Clinit(CompilationResult compilationResult) {
 		super(compilationResult);
-		modifiers = 0;
-		selector = TypeConstants.CLINIT;
+		this.modifiers = 0;
+		this.selector = TypeConstants.CLINIT;
 	}
 
 	public void analyseCode(
@@ -45,7 +50,7 @@ public class Clinit extends AbstractMethodDeclaration {
 		InitializationFlowContext staticInitializerFlowContext,
 		FlowInfo flowInfo) {
 
-		if (ignoreFurtherInvestigation)
+		if (this.ignoreFurtherInvestigation)
 			return;
 		try {
 			ExceptionHandlingFlowContext clinitContext =
@@ -53,31 +58,38 @@ public class Clinit extends AbstractMethodDeclaration {
 					staticInitializerFlowContext.parent,
 					this,
 					Binding.NO_EXCEPTIONS,
-					scope,
+					staticInitializerFlowContext,
+					this.scope,
 					FlowInfo.DEAD_END);
 
 			// check for missing returning path
-			if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) == 0) {
+			if ((flowInfo.tagBits & FlowInfo.UNREACHABLE_OR_DEAD) == 0) {
 				this.bits |= ASTNode.NeedFreeReturn;
 			}
 
 			// check missing blank final field initializations
 			flowInfo = flowInfo.mergedWith(staticInitializerFlowContext.initsOnReturn);
-			FieldBinding[] fields = scope.enclosingSourceType().fields();
+			FieldBinding[] fields = this.scope.enclosingSourceType().fields();
 			for (int i = 0, count = fields.length; i < count; i++) {
-				FieldBinding field;
-				if ((field = fields[i]).isStatic()
-					&& field.isFinal()
-					&& (!flowInfo.isDefinitelyAssigned(fields[i]))) {
-					scope.problemReporter().uninitializedBlankFinalField(
-						field,
-						scope.referenceType().declarationOf(field.original()));
-					// can complain against the field decl, since only one <clinit>
+				FieldBinding field = fields[i];
+				if (field.isStatic()) {
+					if (!flowInfo.isDefinitelyAssigned(field)) {
+						if (field.isFinal()) {
+							this.scope.problemReporter().uninitializedBlankFinalField(
+									field,
+									this.scope.referenceType().declarationOf(field.original()));
+							// can complain against the field decl, since only one <clinit>
+						} else if (field.isNonNull()) {
+							this.scope.problemReporter().uninitializedNonNullField(
+									field,
+									this.scope.referenceType().declarationOf(field.original()));
+						}
+					}
 				}
 			}
 			// check static initializers thrown exceptions
 			staticInitializerFlowContext.checkInitializerExceptions(
-				scope,
+				this.scope,
 				clinitContext,
 				flowInfo);
 		} catch (AbortMethod e) {
@@ -94,39 +106,45 @@ public class Clinit extends AbstractMethodDeclaration {
 	public void generateCode(ClassScope classScope, ClassFile classFile) {
 
 		int clinitOffset = 0;
-		if (ignoreFurtherInvestigation) {
+		if (this.ignoreFurtherInvestigation) {
 			// should never have to add any <clinit> problem method
 			return;
 		}
-		try {
-			clinitOffset = classFile.contentsOffset;
-			this.generateCode(classScope, classFile, clinitOffset);
-		} catch (AbortMethod e) {
-			// should never occur
-			// the clinit referenceContext is the type declaration
-			// All clinit problems will be reported against the type: AbortType instead of AbortMethod
-			// reset the contentsOffset to the value before generating the clinit code
-			// decrement the number of method info as well.
-			// This is done in the addProblemMethod and addProblemConstructor for other
-			// cases.
-			if (e.compilationResult == CodeStream.RESTART_IN_WIDE_MODE) {
-				// a branch target required a goto_w, restart code gen in wide mode.
-				try {
+		boolean restart = false;
+		do {
+			try {
+				clinitOffset = classFile.contentsOffset;
+				this.generateCode(classScope, classFile, clinitOffset);
+				restart = false;
+			} catch (AbortMethod e) {
+				// should never occur
+				// the clinit referenceContext is the type declaration
+				// All clinit problems will be reported against the type: AbortType instead of AbortMethod
+				// reset the contentsOffset to the value before generating the clinit code
+				// decrement the number of method info as well.
+				// This is done in the addProblemMethod and addProblemConstructor for other
+				// cases.
+				if (e.compilationResult == CodeStream.RESTART_IN_WIDE_MODE) {
+					// a branch target required a goto_w, restart code gen in wide mode.
 					classFile.contentsOffset = clinitOffset;
 					classFile.methodCount--;
-					classFile.codeStream.resetInWideMode();
-					this.generateCode(classScope, classFile, clinitOffset);
+					classFile.codeStream.resetInWideMode(); // request wide mode
 					// restart method generation
-				} catch (AbortMethod e2) {
+					restart = true;
+				} else if (e.compilationResult == CodeStream.RESTART_CODE_GEN_FOR_UNUSED_LOCALS_MODE) {
 					classFile.contentsOffset = clinitOffset;
 					classFile.methodCount--;
+					classFile.codeStream.resetForCodeGenUnusedLocals();
+					// restart method generation
+					restart = true;
+				} else {
+					// produce a problem method accounting for this fatal error
+					classFile.contentsOffset = clinitOffset;
+					classFile.methodCount--;
+					restart = false;
 				}
-			} else {
-				// produce a problem method accounting for this fatal error
-				classFile.contentsOffset = clinitOffset;
-				classFile.methodCount--;
 			}
-		}
+		} while (restart);
 	}
 
 	/**
@@ -147,7 +165,7 @@ public class Clinit extends AbstractMethodDeclaration {
 		int codeAttributeOffset = classFile.contentsOffset;
 		classFile.generateCodeAttributeHeader();
 		CodeStream codeStream = classFile.codeStream;
-		this.resolve(classScope);
+		resolve(classScope);
 
 		codeStream.reset(this, classFile);
 		TypeDeclaration declaringType = classScope.referenceContext;
@@ -173,21 +191,51 @@ public class Clinit extends AbstractMethodDeclaration {
 			falseLabel.place();
 			codeStream.iconst_0();
 			jumpLabel.place();
-			codeStream.putstatic(this.assertionSyntheticFieldBinding);
+			codeStream.fieldAccess(Opcodes.OPC_putstatic, this.assertionSyntheticFieldBinding, null /* default declaringClass */);
 		}
 		// generate static fields/initializers/enum constants
 		final FieldDeclaration[] fieldDeclarations = declaringType.fields;
-		BlockScope lastInitializerScope = null;
+		int sourcePosition = -1;
+		int remainingFieldCount = 0;
 		if (TypeDeclaration.kind(declaringType.modifiers) == TypeDeclaration.ENUM_DECL) {
-			int enumCount = 0;
-			int remainingFieldCount = 0;
-			if (fieldDeclarations != null) {
+			int enumCount = declaringType.enumConstantsCounter;
+			if (enumCount > ENUM_CONSTANTS_THRESHOLD) {
+				// generate synthetic methods to initialize all the enum constants
+				int begin = -1;
+				int count = 0;
+				if (fieldDeclarations != null) {
+					int max = fieldDeclarations.length;
+					for (int i = 0; i < max; i++) {
+						FieldDeclaration fieldDecl = fieldDeclarations[i];
+						if (fieldDecl.isStatic()) {
+							if (fieldDecl.getKind() == AbstractVariableDeclaration.ENUM_CONSTANT) {
+								if (begin == -1) {
+									begin = i;
+								}
+								count++;
+								if (count > ENUM_CONSTANTS_THRESHOLD) {
+									SyntheticMethodBinding syntheticMethod = declaringType.binding.addSyntheticMethodForEnumInitialization(begin, i);
+									codeStream.invoke(Opcodes.OPC_invokestatic, syntheticMethod, null /* default declaringClass */);
+									begin = i;
+									count = 1;
+								}
+							} else {
+								remainingFieldCount++;
+							}
+						}
+					}
+					if (count != 0) {
+						// add last synthetic method
+						SyntheticMethodBinding syntheticMethod = declaringType.binding.addSyntheticMethodForEnumInitialization(begin, max);
+						codeStream.invoke(Opcodes.OPC_invokestatic, syntheticMethod, null /* default declaringClass */);
+					}
+				}
+			} else if (fieldDeclarations != null) {
 				for (int i = 0, max = fieldDeclarations.length; i < max; i++) {
 					FieldDeclaration fieldDecl = fieldDeclarations[i];
 					if (fieldDecl.isStatic()) {
 						if (fieldDecl.getKind() == AbstractVariableDeclaration.ENUM_CONSTANT) {
 							fieldDecl.generateCode(staticInitializerScope, codeStream);
-							enumCount++;
 						} else {
 							remainingFieldCount++;
 						}
@@ -206,30 +254,34 @@ public class Clinit extends AbstractMethodDeclaration {
 						if (fieldDecl.getKind() == AbstractVariableDeclaration.ENUM_CONSTANT) {
 							codeStream.dup();
 							codeStream.generateInlinedValue(fieldDecl.binding.id);
-							codeStream.getstatic(fieldDecl.binding);
+							codeStream.fieldAccess(Opcodes.OPC_getstatic, fieldDecl.binding, null /* default declaringClass */);
 							codeStream.aastore();
 						}
 					}
 				}
 			}
-			codeStream.putstatic(declaringType.enumValuesSyntheticfield);
+			codeStream.fieldAccess(Opcodes.OPC_putstatic, declaringType.enumValuesSyntheticfield, null /* default declaringClass */);
 			if (remainingFieldCount != 0) {
 				// if fields that are not enum constants need to be generated (static initializer/static field)
-				for (int i = 0, max = fieldDeclarations.length; i < max; i++) {
+				for (int i = 0, max = fieldDeclarations.length; i < max && remainingFieldCount >= 0; i++) {
 					FieldDeclaration fieldDecl = fieldDeclarations[i];
 					switch (fieldDecl.getKind()) {
 						case AbstractVariableDeclaration.ENUM_CONSTANT :
 							break;
 						case AbstractVariableDeclaration.INITIALIZER :
-							if (!fieldDecl.isStatic()) 
+							if (!fieldDecl.isStatic()) {
 								break;
-							lastInitializerScope = ((Initializer) fieldDecl).block.scope;
+							}
+							remainingFieldCount--;
+							sourcePosition = ((Initializer) fieldDecl).block.sourceEnd;
 							fieldDecl.generateCode(staticInitializerScope, codeStream);
 							break;
 						case AbstractVariableDeclaration.FIELD :
-							if (!fieldDecl.binding.isStatic()) 
+							if (!fieldDecl.binding.isStatic()) {
 								break;
-							lastInitializerScope = null;
+							}
+							remainingFieldCount--;
+							sourcePosition = fieldDecl.declarationEnd;
 							fieldDecl.generateCode(staticInitializerScope, codeStream);
 							break;
 					}
@@ -241,22 +293,22 @@ public class Clinit extends AbstractMethodDeclaration {
 					FieldDeclaration fieldDecl = fieldDeclarations[i];
 					switch (fieldDecl.getKind()) {
 						case AbstractVariableDeclaration.INITIALIZER :
-							if (!fieldDecl.isStatic()) 
+							if (!fieldDecl.isStatic())
 								break;
-							lastInitializerScope = ((Initializer) fieldDecl).block.scope;
+							sourcePosition = ((Initializer) fieldDecl).block.sourceEnd;
 							fieldDecl.generateCode(staticInitializerScope, codeStream);
 							break;
 						case AbstractVariableDeclaration.FIELD :
-							if (!fieldDecl.binding.isStatic()) 
+							if (!fieldDecl.binding.isStatic())
 								break;
-							lastInitializerScope = null;
+							sourcePosition = fieldDecl.declarationEnd;
 							fieldDecl.generateCode(staticInitializerScope, codeStream);
 							break;
 					}
 				}
 			}
 		}
-		
+
 		if (codeStream.position == 0) {
 			// do not need to output a Clinit if no bytecodes
 			// so we reset the offset inside the byte array contents.
@@ -269,9 +321,9 @@ public class Clinit extends AbstractMethodDeclaration {
 			if ((this.bits & ASTNode.NeedFreeReturn) != 0) {
 				int before = codeStream.position;
 				codeStream.return_();
-				if (lastInitializerScope != null) {
+				if (sourcePosition != -1) {
 					// expand the last initializer variables to include the trailing return
-					codeStream.updateLastRecordedEndPC(lastInitializerScope, before);
+					codeStream.recordPositionsFrom(before, sourcePosition);
 				}
 			}
 			// Record the end of the clinit: point to the declaration of the class
@@ -296,7 +348,7 @@ public class Clinit extends AbstractMethodDeclaration {
 	}
 
 	public void parseStatements(Parser parser, CompilationUnitDeclaration unit) {
-		//the clinit is filled by hand .... 
+		//the clinit is filled by hand ....
 	}
 
 	public StringBuffer print(int tab, StringBuffer output) {
@@ -329,7 +381,7 @@ public class Clinit extends AbstractMethodDeclaration {
 				this.scope.outerMostClassScope().enclosingSourceType();
 			// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=22334
 			if (!sourceType.isInterface() && !sourceType.isBaseType()) {
-				this.classLiteralSyntheticField = sourceType.addSyntheticFieldForClassLiteral(sourceType, scope);
+				this.classLiteralSyntheticField = sourceType.addSyntheticFieldForClassLiteral(sourceType, this.scope);
 			}
 		}
 	}
