@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2007 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,16 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contributions for
+ *     							bug 343713 - [compiler] bogus line number in constructor of inner class in 1.5 compliance
+ *     							bug 349326 - [1.7] new warning for missing try-with-resources
+ *								bug 186342 - [compiler][null] Using annotations for null checking
+ *								bug 361407 - Resource leak warning when resource is assigned to a field outside of constructor
+ *								bug 368546 - [compiler][resource] Avoid remaining false positives found when compiling the Eclipse SDK
+ *								bug 383690 - [compiler] location of error re uninitialized final field should be aligned
+ *								bug 331649 - [compiler][null] consider null annotations for fields
+ *								bug 383368 - [compiler][null] syntactic null analysis for field references
+ *								bug 400421 - [compiler] Null analysis for fields does not take @com.google.inject.Inject into account
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -31,18 +41,9 @@ public class ConstructorDeclaration extends AbstractMethodDeclaration {
 public ConstructorDeclaration(CompilationResult compilationResult){
 	super(compilationResult);
 }
-
-/** 
- * @see org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration#analyseCode(org.eclipse.jdt.internal.compiler.lookup.ClassScope, org.eclipse.jdt.internal.compiler.flow.InitializationFlowContext, org.eclipse.jdt.internal.compiler.flow.FlowInfo)
- * @deprecated use instead {@link #analyseCode(ClassScope, InitializationFlowContext, FlowInfo, int)}
- */
-public void analyseCode(ClassScope classScope, InitializationFlowContext initializerFlowContext, FlowInfo flowInfo) {
-	analyseCode(classScope, initializerFlowContext, flowInfo, FlowInfo.REACHABLE);
-}
-
 /**
  * The flowInfo corresponds to non-static field initialization infos. It may be unreachable (155423), but still the explicit constructor call must be
- * analysed as reachable, since it will be generated in the end.
+ * analyzed as reachable, since it will be generated in the end.
  */
 public void analyseCode(ClassScope classScope, InitializationFlowContext initializerFlowContext, FlowInfo flowInfo, int initialReachMode) {
 	if (this.ignoreFurtherInvestigation)
@@ -50,7 +51,7 @@ public void analyseCode(ClassScope classScope, InitializationFlowContext initial
 
 	int nonStaticFieldInfoReachMode = flowInfo.reachMode();
 	flowInfo.setReachMode(initialReachMode);
-	
+
 	checkUnused: {
 		MethodBinding constructorBinding;
 		if ((constructorBinding = this.binding) == null) break checkUnused;
@@ -59,24 +60,55 @@ public void analyseCode(ClassScope classScope, InitializationFlowContext initial
 		if (constructorBinding.isPrivate()) {
 			if ((this.binding.declaringClass.tagBits & TagBits.HasNonPrivateConstructor) == 0)
 				break checkUnused; // tolerate as known pattern to block instantiation
-		} else if ((this.binding.declaringClass.tagBits & (TagBits.IsAnonymousType|TagBits.IsLocalType)) != TagBits.IsLocalType) {
+		} else if (!constructorBinding.isOrEnclosedByPrivateType()) {
 			break checkUnused;
+ 		}
+		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=270446, When the AST built is an abridged version
+		// we don't have all tree nodes we would otherwise expect. (see ASTParser.setFocalPosition)
+		if (this.constructorCall == null)
+			break checkUnused; 
+		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=264991, Don't complain about this
+		// constructor being unused if the base class doesn't have a no-arg constructor.
+		// See that a seemingly unused constructor that chains to another constructor with a
+		// this(...) can be flagged as being unused without hesitation.
+		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=265142
+		if (this.constructorCall.accessMode != ExplicitConstructorCall.This) {
+			ReferenceBinding superClass = constructorBinding.declaringClass.superclass();
+			if (superClass == null)
+				break checkUnused;
+			// see if there is a no-arg super constructor
+			MethodBinding methodBinding = superClass.getExactConstructor(Binding.NO_PARAMETERS);
+			if (methodBinding == null)
+				break checkUnused;
+			if (!methodBinding.canBeSeenBy(SuperReference.implicitSuperConstructorCall(), this.scope))
+				break checkUnused;
+			// otherwise default super constructor exists, so go ahead and complain unused.
 		}
 		// complain unused
 		this.scope.problemReporter().unusedPrivateConstructor(this);
 	}
-		
+
 	// check constructor recursion, once all constructor got resolved
-	if (isRecursive(null /*lazy initialized visited list*/)) {				
+	if (isRecursive(null /*lazy initialized visited list*/)) {
 		this.scope.problemReporter().recursiveConstructorInvocation(this.constructorCall);
 	}
-		
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=385780
+	if (this.typeParameters != null  &&
+			!this.scope.referenceCompilationUnit().compilationResult.hasSyntaxError) {
+		for (int i = 0, length = this.typeParameters.length; i < length; ++i) {
+			TypeParameter typeParameter = this.typeParameters[i];
+			if ((typeParameter.binding.modifiers & ExtraCompilerModifiers.AccLocallyUsed) == 0) {
+				this.scope.problemReporter().unusedTypeParameter(typeParameter);						
+			}
+		}
+	}
 	try {
 		ExceptionHandlingFlowContext constructorContext =
 			new ExceptionHandlingFlowContext(
 				initializerFlowContext.parent,
 				this,
 				this.binding.thrownExceptions,
+				initializerFlowContext,
 				this.scope,
 				FlowInfo.DEAD_END);
 		initializerFlowContext.checkInitializerExceptions(
@@ -96,14 +128,10 @@ public void analyseCode(ClassScope classScope, InitializationFlowContext initial
 				}
 			}
 		}
-		
-		// tag parameters as being set
-		if (this.arguments != null) {
-			for (int i = 0, count = this.arguments.length; i < count; i++) {
-				flowInfo.markAsDefinitelyAssigned(this.arguments[i].binding);
-			}
-		}
-		
+
+		// nullity and mark as assigned
+		analyseArguments(flowInfo);
+
 		// propagate to constructor call
 		if (this.constructorCall != null) {
 			// if calling 'this(...)', then flag all non-static fields as definitely
@@ -119,55 +147,93 @@ public void analyseCode(ClassScope classScope, InitializationFlowContext initial
 			}
 			flowInfo = this.constructorCall.analyseCode(this.scope, constructorContext, flowInfo);
 		}
-		
+
 		// reuse the reachMode from non static field info
 		flowInfo.setReachMode(nonStaticFieldInfoReachMode);
 
 		// propagate to statements
 		if (this.statements != null) {
-			boolean didAlreadyComplain = false;
+			boolean enableSyntacticNullAnalysisForFields = this.scope.compilerOptions().enableSyntacticNullAnalysisForFields;
+			int complaintLevel = (nonStaticFieldInfoReachMode & FlowInfo.UNREACHABLE) == 0 ? Statement.NOT_COMPLAINED : Statement.COMPLAINED_FAKE_REACHABLE;
 			for (int i = 0, count = this.statements.length; i < count; i++) {
 				Statement stat = this.statements[i];
-				if (!stat.complainIfUnreachable(flowInfo, this.scope, didAlreadyComplain)) {
+				if ((complaintLevel = stat.complainIfUnreachable(flowInfo, this.scope, complaintLevel, true)) < Statement.COMPLAINED_UNREACHABLE) {
 					flowInfo = stat.analyseCode(this.scope, constructorContext, flowInfo);
-				} else {
-					didAlreadyComplain = true;
+				}
+				if (enableSyntacticNullAnalysisForFields) {
+					constructorContext.expireNullCheckedFieldInfo();
 				}
 			}
 		}
 		// check for missing returning path
-		if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) == 0) {
+		if ((flowInfo.tagBits & FlowInfo.UNREACHABLE_OR_DEAD) == 0) {
 			this.bits |= ASTNode.NeedFreeReturn;
 		}
 
 		// reuse the initial reach mode for diagnosing missing blank finals
-		flowInfo.setReachMode(initialReachMode);		
+		// no, we should use the updated reach mode for diagnosing uninitialized blank finals.
+		// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=235781
+		// flowInfo.setReachMode(initialReachMode);
 
-		// check missing blank final field initializations
+		// check missing blank final field initializations (plus @NonNull)
 		if ((this.constructorCall != null)
 			&& (this.constructorCall.accessMode != ExplicitConstructorCall.This)) {
 			flowInfo = flowInfo.mergedWith(constructorContext.initsOnReturn);
 			FieldBinding[] fields = this.binding.declaringClass.fields();
 			for (int i = 0, count = fields.length; i < count; i++) {
-				FieldBinding field;
-				if ((!(field = fields[i]).isStatic())
-					&& field.isFinal()
-					&& (!flowInfo.isDefinitelyAssigned(fields[i]))) {
-					this.scope.problemReporter().uninitializedBlankFinalField(
-						field,
-						((this.bits & ASTNode.IsDefaultConstructor) != 0) ? (ASTNode) this.scope.referenceType() : this);
+				FieldBinding field = fields[i];
+				if (!field.isStatic() && !flowInfo.isDefinitelyAssigned(field)) {
+					if (field.isFinal()) {
+						this.scope.problemReporter().uninitializedBlankFinalField(
+								field,
+								((this.bits & ASTNode.IsDefaultConstructor) != 0)
+									? (ASTNode) this.scope.referenceType().declarationOf(field.original())
+									: this);
+					} else if (field.isNonNull()) {
+						FieldDeclaration fieldDecl = this.scope.referenceType().declarationOf(field.original());
+						if (!isValueProvidedUsingAnnotation(fieldDecl))
+							this.scope.problemReporter().uninitializedNonNullField(
+								field,
+								((this.bits & ASTNode.IsDefaultConstructor) != 0) 
+									? (ASTNode) fieldDecl
+									: this);
+					}
 				}
 			}
 		}
 		// check unreachable catch blocks
 		constructorContext.complainIfUnusedExceptionHandlers(this);
 		// check unused parameters
-		scope.checkUnusedParameters(binding);
+		this.scope.checkUnusedParameters(this.binding);
+		this.scope.checkUnclosedCloseables(flowInfo, null, null/*don't report against a specific location*/, null);
 	} catch (AbortMethod e) {
 		this.ignoreFurtherInvestigation = true;
 	}
 }
 
+boolean isValueProvidedUsingAnnotation(FieldDeclaration fieldDecl) {
+	// a member field annotated with @Inject is considered to be initialized by the injector 
+	if (fieldDecl.annotations != null) {
+		int length = fieldDecl.annotations.length;
+		for (int i = 0; i < length; i++) {
+			Annotation annotation = fieldDecl.annotations[i];
+			if (annotation.resolvedType.id == TypeIds.T_JavaxInjectInject) {
+				return true; // no concept of "optional"
+			} else if (annotation.resolvedType.id == TypeIds.T_ComGoogleInjectInject) {
+				MemberValuePair[] memberValuePairs = annotation.memberValuePairs();
+				if (memberValuePairs == Annotation.NoValuePairs)
+					return true;
+				for (int j = 0; j < memberValuePairs.length; j++) {
+					// if "optional=false" is specified, don't rely on initialization by the injector:
+					if (CharOperation.equals(memberValuePairs[j].name, TypeConstants.OPTIONAL))
+						return memberValuePairs[j].value instanceof FalseLiteral;
+				}
+			}
+		}
+	}
+	return false;
+}
+
 /**
  * Bytecode generation for a constructor
  *
@@ -187,57 +253,66 @@ public void generateCode(ClassScope classScope, ClassFile classFile) {
 		classFile.addProblemConstructor(this, this.binding, problemsCopy);
 		return;
 	}
-	try {
-		problemResetPC = classFile.contentsOffset;
-		this.internalGenerateCode(classScope, classFile);
-	} catch (AbortMethod e) {
-		if (e.compilationResult == CodeStream.RESTART_IN_WIDE_MODE) {
-			// a branch target required a goto_w, restart code gen in wide mode.
-			try {
+	boolean restart = false;
+	boolean abort = false;
+	do {
+		try {
+			problemResetPC = classFile.contentsOffset;
+			internalGenerateCode(classScope, classFile);
+			restart = false;
+		} catch (AbortMethod e) {
+			if (e.compilationResult == CodeStream.RESTART_IN_WIDE_MODE) {
+				// a branch target required a goto_w, restart code gen in wide mode.
 				classFile.contentsOffset = problemResetPC;
 				classFile.methodCount--;
-				classFile.codeStream.resetInWideMode();
-				this.internalGenerateCode(classScope, classFile); // restart method generation
-			} catch (AbortMethod e2) {
-				int problemsLength;
-				CategorizedProblem[] problems =
-					this.scope.referenceCompilationUnit().compilationResult.getAllProblems();
-				CategorizedProblem[] problemsCopy = new CategorizedProblem[problemsLength = problems.length];
-				System.arraycopy(problems, 0, problemsCopy, 0, problemsLength);
-				classFile.addProblemConstructor(this, this.binding, problemsCopy, problemResetPC);
+				classFile.codeStream.resetInWideMode(); // request wide mode
+				restart = true;
+			} else if (e.compilationResult == CodeStream.RESTART_CODE_GEN_FOR_UNUSED_LOCALS_MODE) {
+				classFile.contentsOffset = problemResetPC;
+				classFile.methodCount--;
+				classFile.codeStream.resetForCodeGenUnusedLocals();
+				restart = true;
+			} else {
+				restart = false;
+				abort = true;
 			}
-		} else {
-			int problemsLength;
-			CategorizedProblem[] problems =
-				this.scope.referenceCompilationUnit().compilationResult.getAllProblems();
-			CategorizedProblem[] problemsCopy = new CategorizedProblem[problemsLength = problems.length];
-			System.arraycopy(problems, 0, problemsCopy, 0, problemsLength);
-			classFile.addProblemConstructor(this, this.binding, problemsCopy, problemResetPC);
 		}
+	} while (restart);
+	if (abort) {
+		int problemsLength;
+		CategorizedProblem[] problems =
+				this.scope.referenceCompilationUnit().compilationResult.getAllProblems();
+		CategorizedProblem[] problemsCopy = new CategorizedProblem[problemsLength = problems.length];
+		System.arraycopy(problems, 0, problemsCopy, 0, problemsLength);
+		classFile.addProblemConstructor(this, this.binding, problemsCopy, problemResetPC);
 	}
 }
 
 public void generateSyntheticFieldInitializationsIfNecessary(MethodScope methodScope, CodeStream codeStream, ReferenceBinding declaringClass) {
 	if (!declaringClass.isNestedType()) return;
-	
+
 	NestedTypeBinding nestedType = (NestedTypeBinding) declaringClass;
 
 	SyntheticArgumentBinding[] syntheticArgs = nestedType.syntheticEnclosingInstances();
-	for (int i = 0, max = syntheticArgs == null ? 0 : syntheticArgs.length; i < max; i++) {
-		SyntheticArgumentBinding syntheticArg;
-		if ((syntheticArg = syntheticArgs[i]).matchingField != null) {
-			codeStream.aload_0();
-			codeStream.load(syntheticArg);
-			codeStream.putfield(syntheticArg.matchingField);
+	if (syntheticArgs != null) {
+		for (int i = 0, max = syntheticArgs.length; i < max; i++) {
+			SyntheticArgumentBinding syntheticArg;
+			if ((syntheticArg = syntheticArgs[i]).matchingField != null) {
+				codeStream.aload_0();
+				codeStream.load(syntheticArg);
+				codeStream.fieldAccess(Opcodes.OPC_putfield, syntheticArg.matchingField, null /* default declaringClass */);
+			}
 		}
 	}
 	syntheticArgs = nestedType.syntheticOuterLocalVariables();
-	for (int i = 0, max = syntheticArgs == null ? 0 : syntheticArgs.length; i < max; i++) {
-		SyntheticArgumentBinding syntheticArg;
-		if ((syntheticArg = syntheticArgs[i]).matchingField != null) {
-			codeStream.aload_0();
-			codeStream.load(syntheticArg);
-			codeStream.putfield(syntheticArg.matchingField);
+	if (syntheticArgs != null) {
+		for (int i = 0, max = syntheticArgs.length; i < max; i++) {
+			SyntheticArgumentBinding syntheticArg;
+			if ((syntheticArg = syntheticArgs[i]).matchingField != null) {
+				codeStream.aload_0();
+				codeStream.load(syntheticArg);
+				codeStream.fieldAccess(Opcodes.OPC_putfield, syntheticArg.matchingField, null /* default declaringClass */);
+			}
 		}
 	}
 }
@@ -245,9 +320,9 @@ public void generateSyntheticFieldInitializationsIfNecessary(MethodScope methodS
 private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 	classFile.generateMethodInfoHeader(this.binding);
 	int methodAttributeOffset = classFile.contentsOffset;
-	int attributeNumber = classFile.generateMethodInfoAttribute(this.binding);
+	int attributeNumber = classFile.generateMethodInfoAttributes(this.binding);
 	if ((!this.binding.isNative()) && (!this.binding.isAbstract())) {
-		
+
 		TypeDeclaration declaringType = classScope.referenceContext;
 		int codeAttributeOffset = classFile.contentsOffset;
 		classFile.generateCodeAttributeHeader();
@@ -261,32 +336,34 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 		int argSlotSize = 1 + enumOffset; // this==aload0
 
 		if (declaringClass.isNestedType()){
-			NestedTypeBinding nestedType = (NestedTypeBinding) declaringClass;
-			this.scope.extraSyntheticArguments = nestedType.syntheticOuterLocalVariables();
+			this.scope.extraSyntheticArguments = declaringClass.syntheticOuterLocalVariables();
 			this.scope.computeLocalVariablePositions(// consider synthetic arguments if any
-				nestedType.enclosingInstancesSlotSize + 1 + enumOffset,
+					declaringClass.getEnclosingInstancesSlotSize() + 1 + enumOffset,
 				codeStream);
-			argSlotSize += nestedType.enclosingInstancesSlotSize;
-			argSlotSize += nestedType.outerLocalVariablesSlotSize;
+			argSlotSize += declaringClass.getEnclosingInstancesSlotSize();
+			argSlotSize += declaringClass.getOuterLocalVariablesSlotSize();
 		} else {
 			this.scope.computeLocalVariablePositions(1 + enumOffset,  codeStream);
 		}
-			
+
 		if (this.arguments != null) {
 			for (int i = 0, max = this.arguments.length; i < max; i++) {
 				// arguments initialization for local variable debug attributes
 				LocalVariableBinding argBinding;
 				codeStream.addVisibleLocalVariable(argBinding = this.arguments[i].binding);
 				argBinding.recordInitializationStartPC(0);
-				TypeBinding argType;
-				if ((argType = argBinding.type) == TypeBinding.LONG || (argType == TypeBinding.DOUBLE)) {
-					argSlotSize += 2;
-				} else {
-					argSlotSize++;
+				switch(argBinding.type.id) {
+					case TypeIds.T_long :
+					case TypeIds.T_double :
+						argSlotSize += 2;
+						break;
+					default :
+						argSlotSize++;
+						break;
 				}
 			}
 		}
-		
+
 		MethodScope initializerScope = declaringType.initializerScope;
 		initializerScope.computeLocalVariablePositions(argSlotSize, codeStream); // offset by the argument size (since not linked to method scope)
 
@@ -297,6 +374,7 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 
 		if (needFieldInitializations && preInitSyntheticFields){
 			generateSyntheticFieldInitializationsIfNecessary(this.scope, codeStream, declaringClass);
+			codeStream.recordPositionsFrom(0, this.bodyStart);
 		}
 		// generate constructor call
 		if (this.constructorCall != null) {
@@ -333,7 +411,11 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 		// local variable attributes
 		codeStream.exitUserScope(this.scope);
 		codeStream.recordPositionsFrom(0, this.bodyEnd);
-		classFile.completeCodeAttribute(codeAttributeOffset);
+		try {
+			classFile.completeCodeAttribute(codeAttributeOffset);
+		} catch(NegativeArraySizeException e) {
+			throw new AbortMethod(this.scope.referenceCompilationUnit().compilationResult, null);
+		}
 		attributeNumber++;
 		if ((codeStream instanceof StackMapFrameCodeStream)
 				&& needFieldInitializations
@@ -341,7 +423,7 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 			((StackMapFrameCodeStream) codeStream).resetSecretLocals();
 		}
 	}
-	classFile.completeMethodInfo(methodAttributeOffset, attributeNumber);
+	classFile.completeMethodInfo(this.binding, methodAttributeOffset, attributeNumber);
 }
 
 public boolean isConstructor() {
@@ -369,9 +451,10 @@ public boolean isRecursive(ArrayList visited) {
 			|| !this.constructorCall.binding.isValidBinding()) {
 		return false;
 	}
-	
-	ConstructorDeclaration targetConstructor = 
+
+	ConstructorDeclaration targetConstructor =
 		((ConstructorDeclaration)this.scope.referenceType().declarationOf(this.constructorCall.binding.original()));
+	if (targetConstructor == null) return false; // https://bugs.eclipse.org/bugs/show_bug.cgi?id=358762
 	if (this == targetConstructor) return true; // direct case
 
 	if (visited == null) { // lazy allocation
@@ -390,7 +473,7 @@ public void parseStatements(Parser parser, CompilationUnitDeclaration unit) {
 	if (((this.bits & ASTNode.IsDefaultConstructor) != 0) && this.constructorCall == null){
 		this.constructorCall = SuperReference.implicitSuperConstructorCall();
 		this.constructorCall.sourceStart = this.sourceStart;
-		this.constructorCall.sourceEnd = this.sourceEnd; 
+		this.constructorCall.sourceEnd = this.sourceEnd;
 		return;
 	}
 	parser.parse(this, unit, false);
@@ -421,16 +504,16 @@ public void resolveJavadoc() {
 		if (this.binding.declaringClass != null && !this.binding.declaringClass.isLocalType()) {
 			// Set javadoc visibility
 			int javadocVisibility = this.binding.modifiers & ExtraCompilerModifiers.AccVisibilityMASK;
-			ClassScope classScope = scope.classScope();
+			ClassScope classScope = this.scope.classScope();
 			ProblemReporter reporter = this.scope.problemReporter();
 			int severity = reporter.computeSeverity(IProblem.JavadocMissing);
 			if (severity != ProblemSeverities.Ignore) {
-				if (classScope != null) {			
+				if (classScope != null) {
 					javadocVisibility = Util.computeOuterMostVisibility(classScope.referenceType(), javadocVisibility);
 				}
 				int javadocModifiers = (this.binding.modifiers & ~ExtraCompilerModifiers.AccVisibilityMASK) | javadocVisibility;
-				reporter.javadocMissing(this.sourceStart, this.sourceEnd, severity, javadocModifiers);				
-			}			
+				reporter.javadocMissing(this.sourceStart, this.sourceEnd, severity, javadocModifiers);
+			}
 		}
 	}
 }
@@ -463,15 +546,15 @@ public void resolveStatements() {
 			this.constructorCall = null;
 		} else {
 			this.constructorCall.resolve(this.scope);
-		}	
+		}
 	}
 	if ((this.modifiers & ExtraCompilerModifiers.AccSemicolonBody) != 0) {
-		this.scope.problemReporter().methodNeedBody(this);		
+		this.scope.problemReporter().methodNeedBody(this);
 	}
 	super.resolveStatements();
 }
 
-public void traverse(ASTVisitor visitor,	ClassScope classScope) {
+public void traverse(ASTVisitor visitor, ClassScope classScope) {
 	if (visitor.visit(this, classScope)) {
 		if (this.javadoc != null) {
 			this.javadoc.traverse(visitor, this.scope);
@@ -486,7 +569,7 @@ public void traverse(ASTVisitor visitor,	ClassScope classScope) {
 			for (int i = 0; i < typeParametersLength; i++) {
 				this.typeParameters[i].traverse(visitor, this.scope);
 			}
-		}			
+		}
 		if (this.arguments != null) {
 			int argumentLength = this.arguments.length;
 			for (int i = 0; i < argumentLength; i++)
@@ -509,5 +592,5 @@ public void traverse(ASTVisitor visitor,	ClassScope classScope) {
 }
 public TypeParameter[] typeParameters() {
     return this.typeParameters;
-}		
+}
 }
