@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,12 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contributions for
+ *								bug 186342 - [compiler][null] Using annotations for null checking
+ *								bug 331649 - [compiler][null] consider null annotations for fields
+ *								bug 383368 - [compiler][null] syntactic null analysis for field references
+ *								bug 382069 - [null] Make the null analysis consider JUnit's assertNotNull similarly to assertions
+ *								bug 403086 - [compiler][null] include the effect of 'assert' in syntactic null analysis for fields
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -24,13 +30,69 @@ public class EqualExpression extends BinaryExpression {
 	}
 	private void checkNullComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse) {
 
-		LocalVariableBinding local = this.left.localVariableBinding();
-		if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
-			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, this.right.nullStatus(flowInfo), this.left);
+		// collect null status of child nodes:
+		int rightStatus = this.right.nullStatus(flowInfo, flowContext);
+		int leftStatus = this.left.nullStatus(flowInfo, flowContext);
+
+		boolean leftNonNullChecked = false;
+		boolean rightNonNullChecked = false;
+
+		// check if either is a non-local expression known to be nonnull and compared to null, candidates are
+		// - method/field annotated @NonNull
+		// - allocation expression, some literals, this reference (see inside expressionNonNullComparison(..))
+		// these checks do not leverage the flowInfo.
+		boolean checkEquality = ((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL;
+		if (leftStatus == FlowInfo.NON_NULL && rightStatus == FlowInfo.NULL) {
+			leftNonNullChecked = scope.problemReporter().expressionNonNullComparison(this.left, checkEquality);
+		} else if (leftStatus == FlowInfo.NULL && rightStatus == FlowInfo.NON_NULL) {
+			rightNonNullChecked = scope.problemReporter().expressionNonNullComparison(this.right, checkEquality);
 		}
-		local = this.right.localVariableBinding();
-		if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
-			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, this.left.nullStatus(flowInfo), this.right);
+		
+		boolean contextualCheckEquality = checkEquality ^ ((flowContext.tagBits & FlowContext.INSIDE_NEGATION) != 0);
+		// perform flowInfo-based checks for variables and record info for syntactic null analysis for fields:
+		if (!leftNonNullChecked) {
+			LocalVariableBinding local = this.left.localVariableBinding();
+			if (local != null) {
+				if ((local.type.tagBits & TagBits.IsBaseType) == 0) {
+					checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, rightStatus, this.left);
+				}
+			} else if (this.left instanceof Reference
+							&& ((!contextualCheckEquality && rightStatus == FlowInfo.NULL) 
+									|| (contextualCheckEquality && rightStatus == FlowInfo.NON_NULL))
+							&& scope.compilerOptions().enableSyntacticNullAnalysisForFields)
+			{
+				FieldBinding field = ((Reference)this.left).lastFieldBinding();
+				if (field != null && (field.type.tagBits & TagBits.IsBaseType) == 0) {
+					flowContext.recordNullCheckedFieldReference((Reference) this.left, 1);
+				}
+			}
+		}
+		if (!rightNonNullChecked) {
+			LocalVariableBinding local = this.right.localVariableBinding();
+			if (local != null) { 
+				if ((local.type.tagBits & TagBits.IsBaseType) == 0) {
+					checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, leftStatus, this.right);
+				}
+			} else if (this.right instanceof Reference
+							&& ((!contextualCheckEquality && leftStatus == FlowInfo.NULL) 
+									|| (contextualCheckEquality && leftStatus == FlowInfo.NON_NULL))
+							&& scope.compilerOptions().enableSyntacticNullAnalysisForFields) 
+			{
+				FieldBinding field = ((Reference)this.right).lastFieldBinding();
+				if (field != null && (field.type.tagBits & TagBits.IsBaseType) == 0) {
+					flowContext.recordNullCheckedFieldReference((Reference) this.right, 1);
+				}				
+			}
+		}
+
+		// handle reachability:
+		if (leftNonNullChecked || rightNonNullChecked) {
+			// above checks have not propagated unreachable into the corresponding branch, do it now:
+			if (checkEquality) {
+				initsWhenTrue.setReachMode(FlowInfo.UNREACHABLE_BY_NULLANALYSIS);
+			} else {
+				initsWhenFalse.setReachMode(FlowInfo.UNREACHABLE_BY_NULLANALYSIS);
+			}
 		}
 	}
 	private void checkVariableComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse, LocalVariableBinding local, int nullStatus, Expression reference) {
@@ -361,6 +423,7 @@ public class EqualExpression extends BinaryExpression {
 		// default case
 		this.left.generateCode(currentScope, codeStream, valueRequired);
 		this.right.generateCode(currentScope, codeStream, valueRequired);
+		int pc = codeStream.position;
 		if (valueRequired) {
 			if (falseLabel == null) {
 				if (trueLabel != null) {
@@ -376,8 +439,7 @@ public class EqualExpression extends BinaryExpression {
 				}
 			}
 		}
-		// reposition the endPC
-		codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+		codeStream.recordPositionsFrom(pc, this.sourceEnd);
 	}
 	/**
 	 * Boolean generation for == with non-boolean operands
@@ -812,7 +874,15 @@ public class EqualExpression extends BinaryExpression {
 			// check whether comparing identical expressions
 			Binding leftDirect = Expression.getDirectBinding(this.left);
 			if (leftDirect != null && leftDirect == Expression.getDirectBinding(this.right)) {
-				scope.problemReporter().comparingIdenticalExpressions(this);
+				if (leftTypeID != TypeIds.T_double && leftTypeID != TypeIds.T_float
+						&&(!(this.right instanceof Assignment))) // https://bugs.eclipse.org/bugs/show_bug.cgi?id=281776
+					scope.problemReporter().comparingIdenticalExpressions(this);
+			} else if (this.constant != Constant.NotAConstant) {
+				// https://bugs.eclipse.org/bugs/show_bug.cgi?id=276740
+				int operator = (this.bits & OperatorMASK) >> OperatorSHIFT;
+				if ((operator == EQUAL_EQUAL && this.constant == BooleanConstant.fromValue(true))
+						|| (operator == NOT_EQUAL && this.constant == BooleanConstant.fromValue(false)))
+					scope.problemReporter().comparingIdenticalExpressions(this);
 			}
 			return this.resolvedType = TypeBinding.BOOLEAN;
 		}
@@ -848,7 +918,9 @@ public class EqualExpression extends BinaryExpression {
 			// check whether comparing identical expressions
 			Binding leftDirect = Expression.getDirectBinding(this.left);
 			if (leftDirect != null && leftDirect == Expression.getDirectBinding(this.right)) {
-				scope.problemReporter().comparingIdenticalExpressions(this);
+				if (!(this.right instanceof Assignment)) {
+					scope.problemReporter().comparingIdenticalExpressions(this);
+				}
 			}
 			return this.resolvedType = TypeBinding.BOOLEAN;
 		}
