@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2008 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,13 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann <stephan@cs.tu-berlin.de> - Contributions for
+ *								bug 185682 - Increment/decrement operators mark local variables as read
+ *								bug 331649 - [compiler][null] consider null annotations for fields
+ *								bug 383368 - [compiler][null] syntactic null analysis for field references
+ *								Bug 412203 - [compiler] Internal compiler error: java.lang.IllegalArgumentException: info cannot be null
+ *     Jesper S Moller - Contributions for
+ *								Bug 378674 - "The method can be declared as static" is wrong
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -14,16 +21,19 @@ import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.internal.compiler.ASTVisitor;
 import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
+import org.eclipse.jdt.internal.compiler.codegen.Opcodes;
 import org.eclipse.jdt.internal.compiler.flow.FlowContext;
 import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
-import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.eclipse.jdt.internal.compiler.impl.Constant;
 import org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
 import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
 import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
+import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
 import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
 import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
+import org.eclipse.jdt.internal.compiler.lookup.MissingTypeBinding;
+import org.eclipse.jdt.internal.compiler.lookup.ProblemFieldBinding;
 import org.eclipse.jdt.internal.compiler.lookup.ProblemReasons;
 import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
 import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
@@ -32,6 +42,7 @@ import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
 import org.eclipse.jdt.internal.compiler.lookup.TagBits;
 import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
 import org.eclipse.jdt.internal.compiler.lookup.TypeIds;
+import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;
 
 public class FieldReference extends Reference implements InvocationSite {
 
@@ -40,13 +51,12 @@ public class FieldReference extends Reference implements InvocationSite {
 	public Expression receiver;
 	public char[] token;
 	public FieldBinding binding;															// exact binding resulting from lookup
-	protected FieldBinding codegenBinding;									// actual binding used for code generation (if no synthetic accessor)
 	public MethodBinding[] syntheticAccessors; // [0]=read accessor [1]=write accessor
-	
+
 	public long nameSourcePosition; //(start<<32)+end
-	public TypeBinding receiverType;
+	public TypeBinding actualReceiverType;
 	public TypeBinding genericCast;
-	
+
 public FieldReference(char[] source, long pos) {
 	this.token = source;
 	this.nameSourcePosition = pos;
@@ -57,15 +67,17 @@ public FieldReference(char[] source, long pos) {
 
 }
 
-public FlowInfo analyseAssignment(BlockScope currentScope, 	FlowContext flowContext, 	FlowInfo flowInfo, Assignment assignment, boolean isCompound) {
+public FlowInfo analyseAssignment(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, Assignment assignment, boolean isCompound) {
 	// compound assignment extra work
 	if (isCompound) { // check the variable part is initialized if blank final
 		if (this.binding.isBlankFinal()
 			&& this.receiver.isThis()
-			&& currentScope.needBlankFinalFieldInitializationCheck(this.binding)
-			&& (!flowInfo.isDefinitelyAssigned(this.binding))) {
-			currentScope.problemReporter().uninitializedBlankFinalField(this.binding, this);
-			// we could improve error msg here telling "cannot use compound assignment on final blank field"
+			&& currentScope.needBlankFinalFieldInitializationCheck(this.binding)) {
+			FlowInfo fieldInits = flowContext.getInitsForFinalBlankInitializationCheck(this.binding.declaringClass.original(), flowInfo);
+			if (!fieldInits.isDefinitelyAssigned(this.binding)) {
+				currentScope.problemReporter().uninitializedBlankFinalField(this.binding, this);
+				// we could improve error msg here telling "cannot use compound assignment on final blank field"
+			}
 		}
 		manageSyntheticAccessIfNecessary(currentScope, flowInfo, true /*read-access*/);
 	}
@@ -82,7 +94,7 @@ public FlowInfo analyseAssignment(BlockScope currentScope, 	FlowContext flowCont
 	}
 	manageSyntheticAccessIfNecessary(currentScope, flowInfo, false /*write-access*/);
 
-	// check if assigning a final field 
+	// check if assigning a final field
 	if (this.binding.isFinal()) {
 		// in a context where it can be assigned?
 		if (this.binding.isBlankFinal()
@@ -103,6 +115,15 @@ public FlowInfo analyseAssignment(BlockScope currentScope, 	FlowContext flowCont
 			// assigning a final field outside an initializer or constructor or wrong reference
 			currentScope.problemReporter().cannotAssignToFinalField(this.binding, this);
 		}
+	} else if (this.binding.isNonNull()) {
+		// in a context where it can be assigned?
+		if (   !isCompound
+			&& this.receiver.isThis()
+			&& !(this.receiver instanceof QualifiedThisReference)
+			&& this.receiver.resolvedType == this.binding.declaringClass // inherited fields are not tracked here
+			&& ((this.receiver.bits & ASTNode.ParenthesizedMASK) == 0)) { // (this).x is forbidden
+			flowInfo.markAsDefinitelyAssigned(this.binding);
+		}		
 	}
 	return flowInfo;
 }
@@ -117,41 +138,48 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 	if (nonStatic) {
 		this.receiver.checkNPE(currentScope, flowContext, flowInfo);
 	}
-	
+
 	if (valueRequired || currentScope.compilerOptions().complianceLevel >= ClassFileConstants.JDK1_4) {
 		manageSyntheticAccessIfNecessary(currentScope, flowInfo, true /*read-access*/);
 	}
 	return flowInfo;
 }
 
+public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo) {
+	if (flowContext.isNullcheckedFieldAccess(this)) {
+		return true; // enough seen
+	}
+	return checkNullableFieldDereference(scope, this.binding, this.nameSourcePosition);
+}
+
 /**
  * @see org.eclipse.jdt.internal.compiler.ast.Expression#computeConversion(org.eclipse.jdt.internal.compiler.lookup.Scope, org.eclipse.jdt.internal.compiler.lookup.TypeBinding, org.eclipse.jdt.internal.compiler.lookup.TypeBinding)
  */
 public void computeConversion(Scope scope, TypeBinding runtimeTimeType, TypeBinding compileTimeType) {
 	if (runtimeTimeType == null || compileTimeType == null)
-		return;		
+		return;
 	// set the generic cast after the fact, once the type expectation is fully known (no need for strict cast)
 	if (this.binding != null && this.binding.isValidBinding()) {
 		FieldBinding originalBinding = this.binding.original();
 		TypeBinding originalType = originalBinding.type;
 	    // extra cast needed if field type is type variable
 		if (originalType.leafComponentType().isTypeVariable()) {
-	    	TypeBinding targetType = (!compileTimeType.isBaseType() && runtimeTimeType.isBaseType()) 
+	    	TypeBinding targetType = (!compileTimeType.isBaseType() && runtimeTimeType.isBaseType())
 	    		? compileTimeType  // unboxing: checkcast before conversion
 	    		: runtimeTimeType;
 	        this.genericCast = originalBinding.type.genericCast(targetType);
 	        if (this.genericCast instanceof ReferenceBinding) {
 				ReferenceBinding referenceCast = (ReferenceBinding) this.genericCast;
 				if (!referenceCast.canBeSeenBy(scope)) {
-		        	scope.problemReporter().invalidType(this, 
+		        	scope.problemReporter().invalidType(this,
 		        			new ProblemReferenceBinding(
 								CharOperation.splitOn('.', referenceCast.shortReadableName()),
 								referenceCast,
 								ProblemReasons.NotVisible));
 				}
-	        }			        
+	        }
 		}
-	} 	
+	}
 	super.computeConversion(scope, runtimeTimeType, compileTimeType);
 }
 
@@ -161,17 +189,11 @@ public FieldBinding fieldBinding() {
 
 public void generateAssignment(BlockScope currentScope, CodeStream codeStream, Assignment assignment, boolean valueRequired) {
 	int pc = codeStream.position;
-	this.receiver.generateCode(
-		currentScope,
-		codeStream,
-		!this.codegenBinding.isStatic());
+	FieldBinding codegenBinding = this.binding.original();
+	this.receiver.generateCode(currentScope, codeStream, !codegenBinding.isStatic());
 	codeStream.recordPositionsFrom(pc, this.sourceStart);
 	assignment.expression.generateCode(currentScope, codeStream, true);
-	fieldStore(
-		codeStream,
-		this.codegenBinding,
-		this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.WRITE],
-		valueRequired);
+	fieldStore(currentScope, codeStream, codegenBinding, this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.WRITE], this.actualReceiverType, this.receiver.isImplicitThis(), valueRequired);
 	if (valueRequired) {
 		codeStream.generateImplicitConversion(assignment.implicitConversion);
 	}
@@ -194,9 +216,10 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 		codeStream.recordPositionsFrom(pc, this.sourceStart);
 		return;
 	}
-	boolean isStatic = this.codegenBinding.isStatic();
+	FieldBinding codegenBinding = this.binding.original();
+	boolean isStatic = codegenBinding.isStatic();
 	boolean isThisReceiver = this.receiver instanceof ThisReference;
-	Constant fieldConstant = this.codegenBinding.constant();
+	Constant fieldConstant = codegenBinding.constant();
 	if (fieldConstant != Constant.NotAConstant) {
 		if (!isThisReceiver) {
 			this.receiver.generateCode(currentScope, codeStream, !isStatic);
@@ -211,13 +234,16 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 		codeStream.recordPositionsFrom(pc, this.sourceStart);
 		return;
 	}
-	if (valueRequired 
+	if (valueRequired
 			|| (!isThisReceiver && currentScope.compilerOptions().complianceLevel >= ClassFileConstants.JDK1_4)
 			|| ((this.implicitConversion & TypeIds.UNBOXING) != 0)
 			|| (this.genericCast != null)) {
 		this.receiver.generateCode(currentScope, codeStream, !isStatic);
+		if ((this.bits & NeedReceiverGenericCast) != 0) {
+			codeStream.checkcast(this.actualReceiverType);
+		}		
 		pc = codeStream.position;
-		if (this.codegenBinding.declaringClass == null) { // array length
+		if (codegenBinding.declaringClass == null) { // array length
 			codeStream.arraylength();
 			if (valueRequired) {
 				codeStream.generateImplicitConversion(this.implicitConversion);
@@ -227,13 +253,14 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 			}
 		} else {
 			if (this.syntheticAccessors == null || this.syntheticAccessors[FieldReference.READ] == null) {
+				TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass(currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
 				if (isStatic) {
-					codeStream.getstatic(this.codegenBinding);
+					codeStream.fieldAccess(Opcodes.OPC_getstatic, codegenBinding, constantPoolDeclaringClass);
 				} else {
-					codeStream.getfield(this.codegenBinding);
+					codeStream.fieldAccess(Opcodes.OPC_getfield, codegenBinding, constantPoolDeclaringClass);
 				}
 			} else {
-				codeStream.invokestatic(this.syntheticAccessors[FieldReference.READ]);
+				codeStream.invoke(Opcodes.OPC_invokestatic, this.syntheticAccessors[FieldReference.READ], null /* default declaringClass */);
 			}
 			// required cast must occur even if no value is required
 			if (this.genericCast != null) codeStream.checkcast(this.genericCast);
@@ -243,7 +270,7 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 				boolean isUnboxing = (this.implicitConversion & TypeIds.UNBOXING) != 0;
 				// conversion only generated if unboxing
 				if (isUnboxing) codeStream.generateImplicitConversion(this.implicitConversion);
-				switch (isUnboxing ? postConversionType(currentScope).id : this.codegenBinding.type.id) {
+				switch (isUnboxing ? postConversionType(currentScope).id : codegenBinding.type.id) {
 					case T_long :
 					case T_double :
 						codeStream.pop2();
@@ -257,14 +284,15 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 		if (isThisReceiver) {
 			if (isStatic){
 				// if no valueRequired, still need possible side-effects of <clinit> invocation, if field belongs to different class
-				if (this.binding.original().declaringClass != this.receiverType.erasure()) {
-					MethodBinding accessor = this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.READ]; 
+				if (this.binding.original().declaringClass != this.actualReceiverType.erasure()) {
+					MethodBinding accessor = this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.READ];
 					if (accessor == null) {
-						codeStream.getstatic(this.codegenBinding);
+						TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass(currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
+						codeStream.fieldAccess(Opcodes.OPC_getstatic, codegenBinding, constantPoolDeclaringClass);
 					} else {
-						codeStream.invokestatic(accessor);
+						codeStream.invoke(Opcodes.OPC_invokestatic, accessor, null /* default declaringClass */);
 					}
-					switch (this.codegenBinding.type.id) {
+					switch (codegenBinding.type.id) {
 						case T_long :
 						case T_double :
 							codeStream.pop2();
@@ -275,34 +303,36 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 				}
 			}
 		} else {
-			this.receiver.generateCode(currentScope, codeStream, !isStatic);				
+			this.receiver.generateCode(currentScope, codeStream, !isStatic);
 			if (!isStatic){
 				codeStream.invokeObjectGetClass(); // perform null check
 				codeStream.pop();
 			}
-		}						
+		}
 	}
 	codeStream.recordPositionsFrom(pc, this.sourceEnd);
 }
 
 public void generateCompoundAssignment(BlockScope currentScope, CodeStream codeStream, Expression expression, int operator, int assignmentImplicitConversion, boolean valueRequired) {
 	boolean isStatic;
-	this.receiver.generateCode(
-		currentScope,
-		codeStream,
-		!(isStatic = this.codegenBinding.isStatic()));
+	// check if compound assignment is the only usage of a private field
+	reportOnlyUselesslyReadPrivateField(currentScope, this.binding, valueRequired);
+	FieldBinding codegenBinding = this.binding.original();
+	this.receiver.generateCode(currentScope, codeStream, !(isStatic = codegenBinding.isStatic()));
 	if (isStatic) {
 		if (this.syntheticAccessors == null || this.syntheticAccessors[FieldReference.READ] == null) {
-			codeStream.getstatic(this.codegenBinding);
+			TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass(currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
+			codeStream.fieldAccess(Opcodes.OPC_getstatic, codegenBinding, constantPoolDeclaringClass);
 		} else {
-			codeStream.invokestatic(this.syntheticAccessors[FieldReference.READ]);
+			codeStream.invoke(Opcodes.OPC_invokestatic, this.syntheticAccessors[FieldReference.READ], null /* default declaringClass */);
 		}
 	} else {
 		codeStream.dup();
 		if (this.syntheticAccessors == null || this.syntheticAccessors[FieldReference.READ] == null) {
-			codeStream.getfield(this.codegenBinding);
+			TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass(currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
+			codeStream.fieldAccess(Opcodes.OPC_getfield, codegenBinding, constantPoolDeclaringClass);
 		} else {
-			codeStream.invokestatic(this.syntheticAccessors[FieldReference.READ]);
+			codeStream.invoke(Opcodes.OPC_invokestatic, this.syntheticAccessors[FieldReference.READ], null /* default declaringClass */);
 		}
 	}
 	int operationTypeID;
@@ -314,7 +344,7 @@ public void generateCompoundAssignment(BlockScope currentScope, CodeStream codeS
 			break;
 		default :
 			if (this.genericCast != null)
-				codeStream.checkcast(this.genericCast);				
+				codeStream.checkcast(this.genericCast);
 			// promote the array reference to the suitable operation type
 			codeStream.generateImplicitConversion(this.implicitConversion);
 			// generate the increment value (will by itself  be promoted to the operation value)
@@ -328,32 +358,30 @@ public void generateCompoundAssignment(BlockScope currentScope, CodeStream codeS
 			// cast the value back to the array reference type
 			codeStream.generateImplicitConversion(assignmentImplicitConversion);
 	}
-	fieldStore(
-		codeStream,
-		this.codegenBinding,
-		this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.WRITE],
-		valueRequired);
+	fieldStore(currentScope, codeStream, codegenBinding, this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.WRITE], this.actualReceiverType, this.receiver.isImplicitThis(), valueRequired);
 	// no need for generic cast as value got dupped
 }
 
 public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream, CompoundAssignment postIncrement, boolean valueRequired) {
 	boolean isStatic;
-	this.receiver.generateCode(
-		currentScope,
-		codeStream,
-		!(isStatic = this.codegenBinding.isStatic()));
+	// check if postIncrement is the only usage of a private field
+	reportOnlyUselesslyReadPrivateField(currentScope, this.binding, valueRequired);
+	FieldBinding codegenBinding = this.binding.original();
+	this.receiver.generateCode(currentScope, codeStream, !(isStatic = codegenBinding.isStatic()));
 	if (isStatic) {
 		if (this.syntheticAccessors == null || this.syntheticAccessors[FieldReference.READ] == null) {
-			codeStream.getstatic(this.codegenBinding);
+			TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass(currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
+			codeStream.fieldAccess(Opcodes.OPC_getstatic, codegenBinding, constantPoolDeclaringClass);
 		} else {
-			codeStream.invokestatic(this.syntheticAccessors[FieldReference.READ]);
+			codeStream.invoke(Opcodes.OPC_invokestatic, this.syntheticAccessors[FieldReference.READ], null /* default declaringClass */);
 		}
 	} else {
 		codeStream.dup();
 		if (this.syntheticAccessors == null || this.syntheticAccessors[FieldReference.READ] == null) {
-			codeStream.getfield(this.codegenBinding);
+			TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass(currentScope, codegenBinding, this.actualReceiverType, this.receiver.isImplicitThis());
+			codeStream.fieldAccess(Opcodes.OPC_getfield, codegenBinding, constantPoolDeclaringClass);
 		} else {
-			codeStream.invokestatic(this.syntheticAccessors[FieldReference.READ]);
+			codeStream.invoke(Opcodes.OPC_invokestatic, this.syntheticAccessors[FieldReference.READ], null /* default declaringClass */);
 		}
 	}
 	TypeBinding operandType;
@@ -361,23 +389,29 @@ public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream
 		codeStream.checkcast(this.genericCast);
 		operandType = this.genericCast;
 	} else {
-		operandType = this.codegenBinding.type;
+		operandType = codegenBinding.type;
 	}	
 	if (valueRequired) {
 		if (isStatic) {
-			if ((operandType == TypeBinding.LONG)
-				|| (operandType == TypeBinding.DOUBLE)) {
-				codeStream.dup2();
-			} else {
-				codeStream.dup();
-			}
+			switch (operandType.id) {
+				case TypeIds.T_long :
+				case TypeIds.T_double :
+					codeStream.dup2();
+					break;
+				default :
+					codeStream.dup();
+					break;
+			}			
 		} else { // Stack:  [owner][old field value]  ---> [old field value][owner][old field value]
-			if ((operandType == TypeBinding.LONG)
-				|| (operandType == TypeBinding.DOUBLE)) {
-				codeStream.dup2_x1();
-			} else {
-				codeStream.dup_x1();
-			}
+			switch (operandType.id) {
+				case TypeIds.T_long :
+				case TypeIds.T_double :
+					codeStream.dup2_x1();
+					break;
+				default :
+					codeStream.dup_x1();
+					break;
+			}			
 		}
 	}
 	codeStream.generateImplicitConversion(this.implicitConversion);		
@@ -387,8 +421,8 @@ public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream
 	codeStream.sendOperator(postIncrement.operator, this.implicitConversion & TypeIds.COMPILE_TYPE_MASK);
 	codeStream.generateImplicitConversion(
 		postIncrement.preAssignImplicitConversion);
-	fieldStore(codeStream, this.codegenBinding, this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.WRITE], false);
-}	
+	fieldStore(currentScope, codeStream, codegenBinding, this.syntheticAccessors == null ? null : this.syntheticAccessors[FieldReference.WRITE], this.actualReceiverType, this.receiver.isImplicitThis(), false);
+}
 
 /**
  * @see org.eclipse.jdt.internal.compiler.lookup.InvocationSite#genericTypeArguments()
@@ -396,6 +430,58 @@ public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream
 public TypeBinding[] genericTypeArguments() {
 	return null;
 }
+
+public boolean isEquivalent(Reference reference) {
+	// only consider field references relative to "this":
+	if (this.receiver.isThis() && !(this.receiver instanceof QualifiedThisReference)) {
+		// current is a simple "this.f1"
+		char[] otherToken = null;
+		// matching 'reference' could be "f1" or "this.f1":
+		if (reference instanceof SingleNameReference) {
+			otherToken = ((SingleNameReference) reference).token;
+		} else if (reference instanceof FieldReference) {
+			FieldReference fr = (FieldReference) reference;
+			if (fr.receiver.isThis() && !(fr.receiver instanceof QualifiedThisReference)) {
+				otherToken = fr.token;
+			}		
+		}
+		return otherToken != null && CharOperation.equals(this.token, otherToken);
+	} else {
+		// search deeper for "this" inside:
+		char[][] thisTokens = getThisFieldTokens(1);
+		if (thisTokens == null) {
+			return false;
+		}
+		// other can be "this.f1.f2", too, or "f1.f2":
+		char[][] otherTokens = null;
+		if (reference instanceof FieldReference) {
+			otherTokens = ((FieldReference) reference).getThisFieldTokens(1);
+		} else if (reference instanceof QualifiedNameReference) {
+			if (((QualifiedNameReference)reference).binding instanceof LocalVariableBinding)
+				return false; // initial variable mismatch: local (from f1.f2) vs. field (from this.f1.f2)
+			otherTokens = ((QualifiedNameReference) reference).tokens;
+		}
+		return CharOperation.equals(thisTokens, otherTokens);
+	}
+}
+
+private char[][] getThisFieldTokens(int nestingCount) {
+	char[][] result = null;
+	if (this.receiver.isThis() && ! (this.receiver instanceof QualifiedThisReference)) {
+		// found an inner-most this-reference, start building the token array:
+		result = new char[nestingCount][];
+		// fill it front to tail while traveling back out:
+		result[0] = this.token;
+	} else if (this.receiver instanceof FieldReference) {
+		result = ((FieldReference)this.receiver).getThisFieldTokens(nestingCount+1);
+		if (result != null) {
+			// front to tail: outermost is last:
+			result[result.length-nestingCount] = this.token;
+		}
+	}
+	return result;
+}
+
 public boolean isSuperAccess() {
 	return this.receiver.isSuper();
 }
@@ -404,40 +490,38 @@ public boolean isTypeAccess() {
 	return this.receiver != null && this.receiver.isTypeReference();
 }
 
+public FieldBinding lastFieldBinding() {
+	return this.binding;
+}
+
 /*
  * No need to emulate access to protected fields since not implicitly accessed
  */
 public void manageSyntheticAccessIfNecessary(BlockScope currentScope, FlowInfo flowInfo, boolean isReadAccess) {
-	if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) != 0)	return;
-
-	// if field from parameterized type got found, use the original field at codegen time
-	this.codegenBinding = this.binding.original();
+	if ((flowInfo.tagBits & FlowInfo.UNREACHABLE_OR_DEAD) != 0)	return;
 	
+	// if field from parameterized type got found, use the original field at codegen time
+	FieldBinding codegenBinding = this.binding.original();
 	if (this.binding.isPrivate()) {
-		if ((currentScope.enclosingSourceType() != this.codegenBinding.declaringClass) 
+		if ((currentScope.enclosingSourceType() != codegenBinding.declaringClass)
 				&& this.binding.constant() == Constant.NotAConstant) {
 			if (this.syntheticAccessors == null)
 				this.syntheticAccessors = new MethodBinding[2];
-			this.syntheticAccessors[isReadAccess ? FieldReference.READ : FieldReference.WRITE] = 
-				((SourceTypeBinding) this.codegenBinding.declaringClass).addSyntheticMethod(this.codegenBinding, isReadAccess);
-			currentScope.problemReporter().needToEmulateFieldAccess(this.codegenBinding, this, isReadAccess);
+			this.syntheticAccessors[isReadAccess ? FieldReference.READ : FieldReference.WRITE] =
+				((SourceTypeBinding) codegenBinding.declaringClass).addSyntheticMethod(codegenBinding, isReadAccess, false /* not super ref in remote type*/);
+			currentScope.problemReporter().needToEmulateFieldAccess(codegenBinding, this, isReadAccess);
 			return;
 		}
-
 	} else if (this.receiver instanceof QualifiedSuperReference) { // qualified super
-
 		// qualified super need emulation always
-		SourceTypeBinding destinationType =
-			(SourceTypeBinding) (((QualifiedSuperReference) this.receiver)
-				.currentCompatibleType);
+		SourceTypeBinding destinationType = (SourceTypeBinding) (((QualifiedSuperReference) this.receiver).currentCompatibleType);
 		if (this.syntheticAccessors == null)
 			this.syntheticAccessors = new MethodBinding[2];
-		this.syntheticAccessors[isReadAccess ? FieldReference.READ : FieldReference.WRITE] = destinationType.addSyntheticMethod(this.codegenBinding, isReadAccess);
-		currentScope.problemReporter().needToEmulateFieldAccess(this.codegenBinding, this, isReadAccess);
+		this.syntheticAccessors[isReadAccess ? FieldReference.READ : FieldReference.WRITE] = destinationType.addSyntheticMethod(codegenBinding, isReadAccess, isSuperAccess());
+		currentScope.problemReporter().needToEmulateFieldAccess(codegenBinding, this, isReadAccess);
 		return;
 
 	} else if (this.binding.isProtected()) {
-
 		SourceTypeBinding enclosingSourceType;
 		if (((this.bits & ASTNode.DepthMASK) != 0)
 			&& this.binding.declaringClass.getPackage()
@@ -448,41 +532,17 @@ public void manageSyntheticAccessIfNecessary(BlockScope currentScope, FlowInfo f
 					(this.bits & ASTNode.DepthMASK) >> ASTNode.DepthSHIFT);
 			if (this.syntheticAccessors == null)
 				this.syntheticAccessors = new MethodBinding[2];
-			this.syntheticAccessors[isReadAccess ? FieldReference.READ : FieldReference.WRITE] = currentCompatibleType.addSyntheticMethod(this.codegenBinding, isReadAccess);
-			currentScope.problemReporter().needToEmulateFieldAccess(this.codegenBinding, this, isReadAccess);
+			this.syntheticAccessors[isReadAccess ? FieldReference.READ : FieldReference.WRITE] = currentCompatibleType.addSyntheticMethod(codegenBinding, isReadAccess, isSuperAccess());
+			currentScope.problemReporter().needToEmulateFieldAccess(codegenBinding, this, isReadAccess);
 			return;
 		}
 	}
-	// if the binding declaring class is not visible, need special action
-	// for runtime compatibility on 1.2 VMs : change the declaring class of the binding
-	// NOTE: from target 1.2 on, field's declaring class is touched if any different from receiver type
-	// and not from Object or implicit static field access.	
-	if (this.binding.declaringClass != this.receiverType
-			&& !this.receiverType.isArrayType()
-			&& this.binding.declaringClass != null // array.length
-			&& this.binding.constant() == Constant.NotAConstant) {
-		CompilerOptions options = currentScope.compilerOptions();
-		if ((options.targetJDK >= ClassFileConstants.JDK1_2
-				&& (options.complianceLevel >= ClassFileConstants.JDK1_4 || !(this.receiver.isImplicitThis() && this.codegenBinding.isStatic()))
-				&& this.binding.declaringClass.id != TypeIds.T_JavaLangObject) // no change for Object fields
-			|| !this.binding.declaringClass.canBeSeenBy(currentScope)) {
-
-			this.codegenBinding =
-				currentScope.enclosingSourceType().getUpdatedFieldBinding(
-					this.codegenBinding,
-					(ReferenceBinding) this.receiverType.erasure());
-		}
-	}		
-}
-
-public int nullStatus(FlowInfo flowInfo) {
-	return FlowInfo.UNKNOWN;
 }
 
 public Constant optimizedBooleanConstant() {
 	switch (this.resolvedType.id) {
 		case T_boolean :
-		case T_JavaLangBoolean :		
+		case T_JavaLangBoolean :
 			return this.constant != Constant.NotAConstant ? this.constant : this.binding.constant();
 		default :
 			return Constant.NotAConstant;
@@ -494,7 +554,7 @@ public Constant optimizedBooleanConstant() {
  */
 public TypeBinding postConversionType(Scope scope) {
 	TypeBinding convertedType = this.resolvedType;
-	if (this.genericCast != null) 
+	if (this.genericCast != null)
 		convertedType = this.genericCast;
 	int runtimeType = (this.implicitConversion & TypeIds.IMPLICIT_CONVERSION_MASK) >> 4;
 	switch (runtimeType) {
@@ -523,7 +583,7 @@ public TypeBinding postConversionType(Scope scope) {
 			convertedType = TypeBinding.DOUBLE;
 			break;
 		default :
-	}		
+	}
 	if ((this.implicitConversion & TypeIds.BOXING) != 0) {
 		convertedType = scope.environment().computeBoxingType(convertedType);
 	}
@@ -537,7 +597,7 @@ public StringBuffer printExpression(int indent, StringBuffer output) {
 public TypeBinding resolveType(BlockScope scope) {
 	// Answer the signature type of the field.
 	// constants are propaged when the field is final
-	// and initialized with a (compile time) constant 
+	// and initialized with a (compile time) constant
 
 	//always ignore receiver cast, since may affect constant pool reference
 	boolean receiverCast = false;
@@ -545,36 +605,60 @@ public TypeBinding resolveType(BlockScope scope) {
 		this.receiver.bits |= ASTNode.DisableUnnecessaryCastCheck; // will check later on
 		receiverCast = true;
 	}
-	this.receiverType = this.receiver.resolveType(scope);
-	if (this.receiverType == null) {
+	this.actualReceiverType = this.receiver.resolveType(scope);
+	if (this.actualReceiverType == null) {
 		this.constant = Constant.NotAConstant;
 		return null;
 	}
 	if (receiverCast) {
 		 // due to change of declaring class with receiver type, only identity cast should be notified
-		if (((CastExpression)this.receiver).expression.resolvedType == this.receiverType) { 
-				scope.problemReporter().unnecessaryCast((CastExpression)this.receiver);		
+		if (((CastExpression)this.receiver).expression.resolvedType == this.actualReceiverType) {
+				scope.problemReporter().unnecessaryCast((CastExpression)this.receiver);
 		}
-	}		
+	}
 	// the case receiverType.isArrayType and token = 'length' is handled by the scope API
-	FieldBinding fieldBinding = this.codegenBinding = this.binding = scope.getField(this.receiverType, this.token, this);
+	FieldBinding fieldBinding = this.binding = scope.getField(this.actualReceiverType, this.token, this);
 	if (!fieldBinding.isValidBinding()) {
 		this.constant = Constant.NotAConstant;
 		if (this.receiver.resolvedType instanceof ProblemReferenceBinding) {
 			// problem already got signaled on receiver, do not report secondary problem
 			return null;
 		}
-		scope.problemReporter().invalidField(this, this.receiverType);
-		return null;
-	}
-	TypeBinding receiverErasure = this.receiverType.erasure();
-	if (receiverErasure instanceof ReferenceBinding) {
-		if (receiverErasure.findSuperTypeOriginatingFrom(fieldBinding.declaringClass) == null) {
-			this.receiverType = fieldBinding.declaringClass; // handle indirect inheritance thru variable secondary bound
+		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=245007 avoid secondary errors in case of
+		// missing super type for anonymous classes ... 
+		ReferenceBinding declaringClass = fieldBinding.declaringClass;
+		boolean avoidSecondary = declaringClass != null &&
+								 declaringClass.isAnonymousType() &&
+								 declaringClass.superclass() instanceof MissingTypeBinding;
+		if (!avoidSecondary) {
+			scope.problemReporter().invalidField(this, this.actualReceiverType);
+		}
+		if (fieldBinding instanceof ProblemFieldBinding) {
+			ProblemFieldBinding problemFieldBinding = (ProblemFieldBinding) fieldBinding;
+			FieldBinding closestMatch = problemFieldBinding.closestMatch;
+			switch(problemFieldBinding.problemId()) {
+				case ProblemReasons.InheritedNameHidesEnclosingName :
+				case ProblemReasons.NotVisible :
+				case ProblemReasons.NonStaticReferenceInConstructorInvocation :
+				case ProblemReasons.NonStaticReferenceInStaticContext :
+					if (closestMatch != null) {
+						fieldBinding = closestMatch;
+					}
+			}
+		}
+		if (!fieldBinding.isValidBinding()) {
+			return null;
 		}
 	}
-	this.receiver.computeConversion(scope, this.receiverType, this.receiverType);
-	if (isFieldUseDeprecated(fieldBinding, scope, (this.bits & ASTNode.IsStrictlyAssigned) !=0)) {
+	// handle indirect inheritance thru variable secondary bound
+	// receiver may receive generic cast, as part of implicit conversion
+	TypeBinding oldReceiverType = this.actualReceiverType;
+	this.actualReceiverType = this.actualReceiverType.getErasureCompatibleType(fieldBinding.declaringClass);
+	this.receiver.computeConversion(scope, this.actualReceiverType, this.actualReceiverType);
+	if (this.actualReceiverType != oldReceiverType && this.receiver.postConversionType(scope) != this.actualReceiverType) { // record need for explicit cast at codegen since receiver could not handle it
+		this.bits |= NeedReceiverGenericCast;
+	}
+	if (isFieldUseDeprecated(fieldBinding, scope, this.bits)) {
 		scope.problemReporter().deprecatedField(fieldBinding, this);
 	}
 	boolean isImplicitThisRcv = this.receiver.isImplicitThis();
@@ -582,13 +666,13 @@ public TypeBinding resolveType(BlockScope scope) {
 	if (fieldBinding.isStatic()) {
 		// static field accessed through receiver? legal but unoptimal (optional warning)
 		if (!(isImplicitThisRcv
-				|| (this.receiver instanceof NameReference 
+				|| (this.receiver instanceof NameReference
 					&& (((NameReference) this.receiver).bits & Binding.TYPE) != 0))) {
 			scope.problemReporter().nonStaticAccessToStaticField(this, fieldBinding);
 		}
 		ReferenceBinding declaringClass = this.binding.declaringClass;
 		if (!isImplicitThisRcv
-				&& declaringClass != this.receiverType
+				&& declaringClass != this.actualReceiverType
 				&& declaringClass.canBeSeenBy(scope)) {
 			scope.problemReporter().indirectAccessToStaticField(this, fieldBinding);
 		}
@@ -602,7 +686,7 @@ public TypeBinding resolveType(BlockScope scope) {
 					&& methodScope.isInsideInitializerOrConstructor()) {
 				scope.problemReporter().enumStaticFieldUsedDuringInitialization(this.binding, this);
 			}
-		}		
+		}
 	}
 	TypeBinding fieldType = fieldBinding.type;
 	if (fieldType != null) {
@@ -613,17 +697,17 @@ public TypeBinding resolveType(BlockScope scope) {
 		if ((fieldType.tagBits & TagBits.HasMissingType) != 0) {
 			scope.problemReporter().invalidType(this, fieldType);
 			return null;
-		}	
+		}
 	}
 	return fieldType;
 }
 
 public void setActualReceiverType(ReferenceBinding receiverType) {
-	// ignored
+	this.actualReceiverType = receiverType;
 }
 
 public void setDepth(int depth) {
-	this.bits &= ~ASTNode.DepthMASK; // flush previous depth if any			
+	this.bits &= ~ASTNode.DepthMASK; // flush previous depth if any
 	if (depth > 0) {
 		this.bits |= (depth & 0xFF) << ASTNode.DepthSHIFT; // encoded on 8 bits
 	}
@@ -639,4 +723,12 @@ public void traverse(ASTVisitor visitor, BlockScope scope) {
 	}
 	visitor.endVisit(this, scope);
 }
+
+public VariableBinding nullAnnotatedVariableBinding() {
+	if (this.binding != null
+			&& ((this.binding.tagBits & (TagBits.AnnotationNonNull|TagBits.AnnotationNullable)) != 0)) {
+		return this.binding;
+	}
+	return null;
+}
 }
