@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,11 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contributions for 
+ *     							bug 319201 - [null] no warning when unboxing SingleNameReference causes NPE
+ *     							bug 349326 - [1.7] new warning for missing try-with-resources
+ *								bug 345305 - [compiler][null] Compiler misidentifies a case of "variable can only be null"
+ *								bug 403147 - [compiler][null] FUP of bug 400761: consolidate interaction between unboxing, NPE, and deferred checking
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -90,8 +95,9 @@ public class ForStatement extends Statement {
 						this.scope,
 						(condLoopContext =
 							new LoopingFlowContext(flowContext, flowInfo, this, null,
-								null, this.scope)),
+								null, this.scope, true)),
 						condInfo);
+				this.condition.checkNPEbyUnboxing(currentScope, flowContext, flowInfo);
 			}
 		}
 
@@ -115,13 +121,15 @@ public class ForStatement extends Statement {
 				actionInfo = condInfo.initsWhenTrue().unconditionalCopy();
 				loopingContext =
 					new LoopingFlowContext(flowContext, flowInfo, this,
-						this.breakLabel, this.continueLabel, this.scope);
+						this.breakLabel, this.continueLabel, this.scope, false);
+						// there is no action guarded by a preTest, so we use preTest=false
+						// to avoid pointless burdens of updating FlowContext.conditionalLevel
 			}
 		}
 		else {
 			loopingContext =
 				new LoopingFlowContext(flowContext, flowInfo, this, this.breakLabel,
-					this.continueLabel, this.scope);
+					this.continueLabel, this.scope, true);
 			FlowInfo initsWhenTrue = condInfo.initsWhenTrue();
 			this.condIfTrueInitStateIndex =
 				currentScope.methodScope().recordInitializationStates(initsWhenTrue);
@@ -131,17 +139,17 @@ public class ForStatement extends Statement {
 				} else {
 					actionInfo = initsWhenTrue.unconditionalCopy();
 					if (isConditionOptimizedFalse){
-						actionInfo.setReachMode(FlowInfo.UNREACHABLE);
+						actionInfo.setReachMode(FlowInfo.UNREACHABLE_OR_DEAD);
 					}
 				}
-			if (this.action.complainIfUnreachable(actionInfo, this.scope, initialComplaintLevel) < Statement.COMPLAINED_UNREACHABLE) {
+			if (this.action.complainIfUnreachable(actionInfo, this.scope, initialComplaintLevel, true) < Statement.COMPLAINED_UNREACHABLE) {
 				actionInfo = this.action.analyseCode(this.scope, loopingContext, actionInfo).unconditionalInits();
 			}
 
 			// code generation can be optimized when no need to continue in the loop
 			if ((actionInfo.tagBits &
 					loopingContext.initsOnContinue.tagBits &
-					FlowInfo.UNREACHABLE) != 0) {
+					FlowInfo.UNREACHABLE_OR_DEAD) != 0) {
 				this.continueLabel = null;
 			}
 			else {
@@ -162,7 +170,7 @@ public class ForStatement extends Statement {
 			if (this.increments != null) {
 				incrementContext =
 					new LoopingFlowContext(flowContext, flowInfo, this, null,
-						null, this.scope);
+						null, this.scope, true);
 				FlowInfo incrementInfo = actionInfo;
 				this.preIncrementsInitStateIndex =
 					currentScope.methodScope().recordInitializationStates(incrementInfo);
@@ -194,7 +202,13 @@ public class ForStatement extends Statement {
 			incrementContext.complainOnDeferredNullChecks(currentScope,
 				actionInfo);
 		}
-
+		if (loopingContext.hasEscapingExceptions()) { // https://bugs.eclipse.org/bugs/show_bug.cgi?id=321926
+			FlowInfo loopbackFlowInfo = flowInfo.copy();
+			if (this.continueLabel != null) {  // we do get to the bottom 
+				loopbackFlowInfo.mergedWith(actionInfo.unconditionalCopy());
+			}
+			loopingContext.simulateThrowAfterLoopBack(loopbackFlowInfo);
+		}
 		//end of loop
 		FlowInfo mergedInfo = FlowInfo.mergedOptimizedBranches(
 				(loopingContext.initsOnBreak.tagBits &
@@ -205,6 +219,17 @@ public class ForStatement extends Statement {
 				exitBranch,
 				isConditionOptimizedFalse,
 				!isConditionTrue /*for(;;){}while(true); unreachable(); */);
+		// Variables initialized only for the purpose of the for loop can be removed for further flow info
+		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=359495
+		if (this.initializations != null) {
+			for (int i = 0; i < this.initializations.length; i++) {
+				Statement init = this.initializations[i];
+				if (init instanceof LocalDeclaration) {
+					LocalVariableBinding binding = ((LocalDeclaration) init).binding;
+					mergedInfo.resetAssignmentInfo(binding);
+				}
+			}
+		}
 		this.mergedInitStateIndex = currentScope.methodScope().recordInitializationStates(mergedInfo);
 		return mergedInfo;
 	}
