@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -38,6 +38,7 @@ public class Javadoc extends ASTNode {
 	public Javadoc(int sourceStart, int sourceEnd) {
 		this.sourceStart = sourceStart;
 		this.sourceEnd = sourceEnd;
+		this.bits |= ASTNode.ResolveJavadoc;
 	}
 	/**
 	 * Returns whether a type can be seen at a given visibility level or not.
@@ -184,6 +185,9 @@ public class Javadoc extends ASTNode {
 	 * Resolve type javadoc
 	 */
 	public void resolve(ClassScope scope) {
+		if ((this.bits & ASTNode.ResolveJavadoc) == 0) {
+			return;
+		}
 		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=247037, @inheritDoc tag cannot
 		// be used in the documentation comment for a class or interface.
 		if (this.inheritedPositions != null) {
@@ -244,6 +248,9 @@ public class Javadoc extends ASTNode {
 	 * Resolve compilation unit javadoc
 	 */
 	public void resolve(CompilationUnitScope unitScope) {
+		if ((this.bits & ASTNode.ResolveJavadoc) == 0) {
+			return;
+		}
 		// Do nothing - This is to mimic the SDK's javadoc tool behavior, which neither
 		// sanity checks nor generates documentation using comments at the CU scope 
 		// (unless the unit happens to be package-info.java - in which case we don't come here.) 
@@ -253,7 +260,9 @@ public class Javadoc extends ASTNode {
 	 * Resolve method javadoc
 	 */
 	public void resolve(MethodScope methScope) {
-
+		if ((this.bits & ASTNode.ResolveJavadoc) == 0) {
+			return;
+		}
 		// get method declaration
 		AbstractMethodDeclaration methDecl = methScope.referenceMethod();
 		boolean overriding = methDecl == null /* field declaration */ || methDecl.binding == null /* compiler error */
@@ -325,7 +334,7 @@ public class Javadoc extends ASTNode {
 		// @param tags
 		CompilerOptions compilerOptions = methScope.compilerOptions();
 		resolveParamTags(methScope, reportMissing, compilerOptions.reportUnusedParameterIncludeDocCommentReference /* considerParamRefAsUsage*/);
-		resolveTypeParameterTags(methScope, reportMissing);
+		resolveTypeParameterTags(methScope, reportMissing && compilerOptions.reportMissingJavadocTagsMethodTypeParameters);
 
 		// @return tags
 		if (this.returnStatement == null) {
@@ -390,7 +399,12 @@ public class Javadoc extends ASTNode {
 					if (scope.enclosingSourceType().isCompatibleWith(fieldRef.actualReceiverType)) {
 						fieldRef.bits |= ASTNode.SuperAccess;
 					}
-					fieldRef.methodBinding = scope.findMethod((ReferenceBinding)fieldRef.actualReceiverType, fieldRef.token, new TypeBinding[0], fieldRef);
+					ReferenceBinding resolvedType = (ReferenceBinding) fieldRef.actualReceiverType;
+					if (CharOperation.equals(resolvedType.sourceName(), fieldRef.token)) {
+						fieldRef.methodBinding = scope.getConstructor(resolvedType, Binding.NO_TYPES, fieldRef);
+					} else {
+						fieldRef.methodBinding = scope.findMethod(resolvedType, fieldRef.token, Binding.NO_TYPES, fieldRef);
+					}
 				}
 			}
 
@@ -570,6 +584,8 @@ public class Javadoc extends ASTNode {
 
 		// If no param tags then report a problem for each declaration type parameter
 		if (parameters != null) {
+			// https://bugs.eclipse.org/bugs/show_bug.cgi?id=324850, avoid secondary errors when <= 1.4 
+			reportMissing = reportMissing && scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5;
 			int typeParametersLength = parameters.length;
 			if (paramTypeParamLength == 0) {
 				if (reportMissing) {
@@ -588,6 +604,11 @@ public class Javadoc extends ASTNode {
 					TypeBinding paramBindind = param.internalResolveType(scope);
 					if (paramBindind != null && paramBindind.isValidBinding()) {
 						if (paramBindind.isTypeVariable()) {
+							// https://bugs.eclipse.org/bugs/show_bug.cgi?id=397888
+							if (scope.compilerOptions().reportUnusedParameterIncludeDocCommentReference) {
+								TypeVariableBinding typeVariableBinding = (TypeVariableBinding) paramBindind;
+								typeVariableBinding.modifiers |= ExtraCompilerModifiers.AccLocallyUsed;
+							}
 							// Verify duplicated tags
 							boolean duplicate = false;
 							for (int j = 0; j < i && !duplicate; j++) {
@@ -819,6 +840,47 @@ public class Javadoc extends ASTNode {
 						}
 					}
 				}
+				if (typeReference instanceof JavadocQualifiedTypeReference && !scope.isDefinedInSameUnit(resolvedType)) {
+					// https://bugs.eclipse.org/bugs/show_bug.cgi?id=222188
+					// partially qualified references from a different CU should be warned
+					char[][] typeRefName = ((JavadocQualifiedTypeReference) typeReference).getTypeName();
+					int skipLength = 0;
+					if (topLevelScope.getCurrentPackage() == resolvedType.getPackage()
+							&& typeRefName.length < computedCompoundName.length) {
+						// https://bugs.eclipse.org/bugs/show_bug.cgi?id=221539: references can be partially qualified
+						// in same package and hence if the package name is not given, ignore package name check
+						skipLength = resolvedType.fPackage.compoundName.length;
+					}
+					boolean valid = true;
+					if (typeRefName.length == computedCompoundName.length - skipLength) {
+						checkQualification: for (int i = 0; i < typeRefName.length; i++) {
+							if (!CharOperation.equals(typeRefName[i], computedCompoundName[i + skipLength])) {
+								valid = false;
+								break checkQualification;
+							}
+						}
+					} else {
+						valid = false;
+					}
+					// report invalid reference
+					if (!valid) {
+						if (scopeModifiers == -1) scopeModifiers = scope.getDeclarationModifiers();
+						scope.problemReporter().javadocInvalidMemberTypeQualification(typeReference.sourceStart, typeReference.sourceEnd, scopeModifiers);
+						return;
+					}
+				}
+			}
+			/*
+			 * https://bugs.eclipse.org/bugs/show_bug.cgi?id=286918
+			 *
+			 * We are concerned only about the Single type references (i.e. without any package) If they are not in default package,
+			 * then report an error. References with qualified yet incorrect names would have already been taken care of.
+			 */
+			if (scope.referenceCompilationUnit().isPackageInfo() && typeReference instanceof JavadocSingleTypeReference) {
+				if (resolvedType.fPackage.compoundName.length > 0) {
+					scope.problemReporter().javadocInvalidReference(typeReference.sourceStart, typeReference.sourceEnd);
+					return; // Not really needed - just in case more code added in future
+				}
 			}
 		}
 	}
