@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2013 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,11 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contributions for
+ *								bug 319201 - [null] no warning when unboxing SingleNameReference causes NPE
+ *								bug 383368 - [compiler][null] syntactic null analysis for field references
+ *								bug 403086 - [compiler][null] include the effect of 'assert' in syntactic null analysis for fields
+ *								bug 403147 - [compiler][null] FUP of bug 400761: consolidate interaction between unboxing, NPE, and deferred checking
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -40,13 +45,17 @@ public class OR_OR_Expression extends BinaryExpression {
 			 // need to be careful of scenario:
 			//		(x || y) || !z, if passing the left info to the right, it would be swapped by the !
 			FlowInfo mergedInfo = this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
+			flowContext.expireNullCheckedFieldInfo();
 			mergedInfo = this.right.analyseCode(currentScope, flowContext, mergedInfo);
+			flowContext.expireNullCheckedFieldInfo();
 			this.mergedInitStateIndex =
 				currentScope.methodScope().recordInitializationStates(mergedInfo);
 			return mergedInfo;
 		}
 
 		FlowInfo leftInfo = this.left.analyseCode(currentScope, flowContext, flowInfo);
+		if ((flowContext.tagBits & FlowContext.INSIDE_NEGATION) == 0)
+			flowContext.expireNullCheckedFieldInfo();
 
 		 // need to be careful of scenario:
 		//		(x || y) || !z, if passing the left info to the right, it would be swapped by the !
@@ -58,13 +67,20 @@ public class OR_OR_Expression extends BinaryExpression {
 		if (isLeftOptimizedTrue){
 			if ((rightInfo.reachMode() & FlowInfo.UNREACHABLE) == 0) {
 				currentScope.problemReporter().fakeReachable(this.right);
-				rightInfo.setReachMode(FlowInfo.UNREACHABLE);
+				rightInfo.setReachMode(FlowInfo.UNREACHABLE_OR_DEAD);
 			}
 		}
 		rightInfo = this.right.analyseCode(currentScope, flowContext, rightInfo);
+		if ((flowContext.tagBits & FlowContext.INSIDE_NEGATION) == 0)
+			flowContext.expireNullCheckedFieldInfo();
+		this.left.checkNPEbyUnboxing(currentScope, flowContext, flowInfo);
+		this.right.checkNPEbyUnboxing(currentScope, flowContext, flowInfo);
+		// The definitely null variables in right info when true should not be missed out while merging
+		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=299900
+		FlowInfo leftInfoWhenTrueForMerging = leftInfo.initsWhenTrue().unconditionalCopy().addPotentialInitializationsFrom(rightInfo.unconditionalInitsWithoutSideEffect());
 		FlowInfo mergedInfo = FlowInfo.conditional(
 					// merging two true initInfos for such a negative case: if ((t && (b = t)) || f) r = b; // b may not have been initialized
-					leftInfo.initsWhenTrue().unconditionalInits().mergedWith(
+				leftInfoWhenTrueForMerging.unconditionalInits().mergedWith(
 						rightInfo.safeInitsWhenTrue().setReachMode(previousMode).unconditionalInits()),
 					rightInfo.initsWhenFalse());
 		this.mergedInitStateIndex =
@@ -98,7 +114,6 @@ public class OR_OR_Expression extends BinaryExpression {
 				codeStream.removeNotDefinitelyAssignedVariables(currentScope, this.mergedInitStateIndex);
 			}
 			codeStream.generateImplicitConversion(this.implicitConversion);
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
 			codeStream.recordPositionsFrom(pc, this.sourceStart);
 			return;
 		}
@@ -142,11 +157,11 @@ public class OR_OR_Expression extends BinaryExpression {
 		if (valueRequired) {
 			if (leftIsConst && leftIsTrue) {
 				codeStream.iconst_1();
-				codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+				codeStream.recordPositionsFrom(codeStream.position, this.left.sourceEnd);
 			} else {
 				if (rightIsConst && rightIsTrue) {
 					codeStream.iconst_1();
-					codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+					codeStream.recordPositionsFrom(codeStream.position, this.left.sourceEnd);
 				} else {
 					codeStream.iconst_0();
 				}
@@ -168,7 +183,7 @@ public class OR_OR_Expression extends BinaryExpression {
 				}
 			}
 			codeStream.generateImplicitConversion(this.implicitConversion);
-			codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+			codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 		} else {
 			trueLabel.place();
 		}
@@ -212,7 +227,7 @@ public class OR_OR_Expression extends BinaryExpression {
 					// need value, e.g. if (a == 1 || ((b = 2) > 0)) {} -> shouldn't initialize 'b' if a==1
 					if (leftIsTrue) {
 						if (valueRequired) codeStream.goto_(trueLabel);
-						codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+						codeStream.recordPositionsFrom(codeStream.position, this.left.sourceEnd);
 						break generateOperands; // no need to generate right operand
 					}
 					if (this.rightInitStateIndex != -1) {
@@ -221,7 +236,7 @@ public class OR_OR_Expression extends BinaryExpression {
 					this.right.generateOptimizedBoolean(currentScope, codeStream, trueLabel, null, valueRequired && !rightIsConst);
 					if (valueRequired && rightIsTrue) {
 						codeStream.goto_(trueLabel);
-						codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+						codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 					}
 				}
 			} else {
@@ -239,9 +254,10 @@ public class OR_OR_Expression extends BinaryExpression {
 								.addDefinitelyAssignedVariables(currentScope, this.rightInitStateIndex);
 					}
 					this.right.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired && !rightIsConst);
+					int pc = codeStream.position;
 					if (valueRequired && rightIsConst && !rightIsTrue) {
 						codeStream.goto_(falseLabel);
-						codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
+						codeStream.recordPositionsFrom(pc, this.sourceEnd);
 					}
 					internalTrueLabel.place();
 				} else {
