@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,7 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contribution for bug 365992 - [builder] [null] Change of nullness for a parameter doesn't trigger a build for the files that call the method
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.classfmt;
 
@@ -207,6 +208,19 @@ public ClassFileReader(byte[] classFileBytes, char[] fileName, boolean fullyInit
 				case ClassFileConstants.NameAndTypeTag :
 					this.constantPoolOffsets[i] = readOffset;
 					readOffset += ClassFileConstants.ConstantNameAndTypeFixedSize;
+					break;
+				case ClassFileConstants.MethodHandleTag :
+					this.constantPoolOffsets[i] = readOffset;
+					readOffset += ClassFileConstants.ConstantMethodHandleFixedSize;
+					break;
+				case ClassFileConstants.MethodTypeTag :
+					this.constantPoolOffsets[i] = readOffset;
+					readOffset += ClassFileConstants.ConstantMethodTypeFixedSize;
+					break;
+				case ClassFileConstants.InvokeDynamicTag :
+					this.constantPoolOffsets[i] = readOffset;
+					readOffset += ClassFileConstants.ConstantInvokeDynamicFixedSize;
+					break;
 			}
 		}
 		// Read and validate access flags
@@ -658,10 +672,15 @@ public char[][][] getMissingTypeNames() {
  * @return int
  */
 public int getModifiers() {
+	int modifiers;
 	if (this.innerInfo != null) {
-		return this.innerInfo.getModifiers() | (this.accessFlags & ClassFileConstants.AccDeprecated);
+		modifiers = this.innerInfo.getModifiers()
+			| (this.accessFlags & ClassFileConstants.AccDeprecated)
+			| (this.accessFlags & ClassFileConstants.AccSynthetic);
+	} else {
+		modifiers = this.accessFlags;
 	}
-	return this.accessFlags;
+	return modifiers;
 }
 
 /**
@@ -969,8 +988,27 @@ private boolean hasStructuralAnnotationChanges(IBinaryAnnotation[] currentAnnota
 		for (int j = 0; j < currentPairsLength; j++) {
 			if (!CharOperation.equals(currentPairs[j].getName(), otherPairs[j].getName()))
 				return true;
-			if (!currentPairs[j].getValue().equals(otherPairs[j].getValue()))
+			final Object value = currentPairs[j].getValue();
+			final Object value2 = otherPairs[j].getValue();
+			if (value instanceof Object[]) {
+				Object[] currentValues = (Object[]) value;
+				if (value2 instanceof Object[]) {
+					Object[] currentValues2 = (Object[]) value2;
+					final int length = currentValues.length;
+					if (length != currentValues2.length) {
+						return true;
+					}
+					for (int n = 0; n < length; n++) {
+						if (!currentValues[n].equals(currentValues2[n])) {
+							return true;
+						}
+					}
+					return false;
+				}
 				return true;
+			} else if (!value.equals(value2)) {
+				return true;
+			}
 		}
 	}
 	return false;
@@ -1033,6 +1071,16 @@ private boolean hasStructuralMethodChanges(MethodInfo currentMethodInfo, MethodI
 		return true;
 	if (hasStructuralAnnotationChanges(currentMethodInfo.getAnnotations(), otherMethodInfo.getAnnotations()))
 		return true;
+	// parameter annotations:
+	int currentAnnotatedParamsCount = currentMethodInfo.getAnnotatedParametersCount();
+	int otherAnnotatedParamsCount = otherMethodInfo.getAnnotatedParametersCount();
+	if (currentAnnotatedParamsCount != otherAnnotatedParamsCount)
+		return true;
+	for (int i=0; i<currentAnnotatedParamsCount; i++) {
+		if (hasStructuralAnnotationChanges(currentMethodInfo.getParameterAnnotations(i), otherMethodInfo.getParameterAnnotations(i)))
+			return true;
+	}
+
 	if (!CharOperation.equals(currentMethodInfo.getSelector(), otherMethodInfo.getSelector()))
 		return true;
 	if (!CharOperation.equals(currentMethodInfo.getMethodDescriptor(), otherMethodInfo.getMethodDescriptor()))
