@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,12 +7,18 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - contributions for
+ *     							bug 337868 - [compiler][model] incomplete support for package-info.java when using SearchableEnvironment
+ *								bug 186342 - [compiler][null] Using annotations for null checking
+ *								bug 365531 - [compiler][null] investigate alternative strategy for internally encoding nullness defaults
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.lookup;
 
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Map;
+import java.util.Set;
 
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.internal.compiler.ClassFilePool;
@@ -22,6 +28,7 @@ import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.env.*;
 import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.eclipse.jdt.internal.compiler.impl.ITypeRequestor;
+import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
 import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
 import org.eclipse.jdt.internal.compiler.util.HashtableOfPackage;
 import org.eclipse.jdt.internal.compiler.util.SimpleLookupTable;
@@ -55,6 +62,10 @@ public class LookupEnvironment implements ProblemReasons, TypeConstants {
 	private SimpleLookupTable uniqueRawTypeBindings;
 	private SimpleLookupTable uniqueWildcardBindings;
 	private SimpleLookupTable uniqueParameterizedGenericMethodBindings;
+	
+	// key is a string with the method selector value is an array of method bindings
+	private SimpleLookupTable uniquePolymorphicMethodBindings;
+	private SimpleLookupTable uniqueGetClassMethodBinding; // https://bugs.eclipse.org/bugs/show_bug.cgi?id=300734
 
 	public CompilationUnitDeclaration unitBeingCompleted = null; // only set while completing units
 	public Object missingClassFileLocation = null; // only set when resolving certain references, to help locating problems
@@ -64,7 +75,13 @@ public class LookupEnvironment implements ProblemReasons, TypeConstants {
 	public MethodBinding arrayClone;
 
 	private ArrayList missingTypes;
+	Set typesBeingConnected;
 	public boolean isProcessingAnnotations = false;
+	public boolean mayTolerateMissingType = false;
+
+	PackageBinding nullableAnnotationPackage;			// the package supposed to contain the Nullable annotation type
+	PackageBinding nonnullAnnotationPackage;			// the package supposed to contain the NonNull annotation type
+	PackageBinding nonnullByDefaultAnnotationPackage;	// the package supposed to contain the NonNullByDefault annotation type
 
 	final static int BUILD_FIELDS_AND_METHODS = 4;
 	final static int BUILD_TYPE_HIERARCHY = 1;
@@ -74,6 +91,7 @@ public class LookupEnvironment implements ProblemReasons, TypeConstants {
 	static final ProblemPackageBinding TheNotFoundPackage = new ProblemPackageBinding(CharOperation.NO_CHAR, NotFound);
 	static final ProblemReferenceBinding TheNotFoundType = new ProblemReferenceBinding(CharOperation.NO_CHAR_CHAR, null, NotFound);
 
+
 public LookupEnvironment(ITypeRequestor typeRequestor, CompilerOptions globalOptions, ProblemReporter problemReporter, INameEnvironment nameEnvironment) {
 	this.typeRequestor = typeRequestor;
 	this.globalOptions = globalOptions;
@@ -88,9 +106,11 @@ public LookupEnvironment(ITypeRequestor typeRequestor, CompilerOptions globalOpt
 	this.uniqueRawTypeBindings = new SimpleLookupTable(3);
 	this.uniqueWildcardBindings = new SimpleLookupTable(3);
 	this.uniqueParameterizedGenericMethodBindings = new SimpleLookupTable(3);
+	this.uniquePolymorphicMethodBindings = new SimpleLookupTable(3);
 	this.missingTypes = null;
 	this.accessRestrictions = new HashMap(3);
 	this.classFilePool = ClassFilePool.newInstance();
+	this.typesBeingConnected = new HashSet();
 }
 
 /**
@@ -120,8 +140,6 @@ public ReferenceBinding askForType(char[][] compoundName) {
 
 ReferenceBinding askForType(PackageBinding packageBinding, char[] name) {
 	if (packageBinding == null) {
-		if (this.defaultPackage == null)
-			return null;
 		packageBinding = this.defaultPackage;
 	}
 	NameEnvironmentAnswer answer = this.nameEnvironment.findType(name, packageBinding.compoundName);
@@ -133,7 +151,13 @@ ReferenceBinding askForType(PackageBinding packageBinding, char[] name) {
 		this.typeRequestor.accept(answer.getBinaryType(), packageBinding, answer.getAccessRestriction());
 	} else if (answer.isCompilationUnit()) {
 		// the type was found as a .java file, try to build it then search the cache
-		this.typeRequestor.accept(answer.getCompilationUnit(), answer.getAccessRestriction());
+		try {
+			this.typeRequestor.accept(answer.getCompilationUnit(), answer.getAccessRestriction());
+		} catch (AbortCompilation abort) {
+			if (CharOperation.equals(name, TypeConstants.PACKAGE_INFO_NAME))
+				return null; // silently, requestor may not be able to handle compilation units (HierarchyResolver)
+			throw abort;
+		}
 	} else if (answer.isSourceType()) {
 		// the type was found as a source model
 		this.typeRequestor.accept(answer.getSourceTypes(), packageBinding, answer.getAccessRestriction());
@@ -256,6 +280,39 @@ public void completeTypeBindings(CompilationUnitDeclaration parsedUnit, boolean
 		parsedUnit.scope.buildFieldsAndMethods();
 	this.unitBeingCompleted = null;
 }
+
+/*
+* Used by other compiler tools which do not start by calling completeTypeBindings()
+* and have more than 1 unit to complete.
+*
+* 1. Connect the type hierarchy for the type bindings created for parsedUnits.
+* 2. Create the field bindings
+* 3. Create the method bindings
+*/
+public void completeTypeBindings(CompilationUnitDeclaration[] parsedUnits, boolean[] buildFieldsAndMethods, int unitCount) {
+	for (int i = 0; i < unitCount; i++) {
+		CompilationUnitDeclaration parsedUnit = parsedUnits[i];
+		if (parsedUnit.scope != null)
+			(this.unitBeingCompleted = parsedUnit).scope.checkAndSetImports();
+	}
+
+	for (int i = 0; i < unitCount; i++) {
+		CompilationUnitDeclaration parsedUnit = parsedUnits[i];
+		if (parsedUnit.scope != null)
+			(this.unitBeingCompleted = parsedUnit).scope.connectTypeHierarchy();
+	}
+
+	for (int i = 0; i < unitCount; i++) {
+		CompilationUnitDeclaration parsedUnit = parsedUnits[i];
+		if (parsedUnit.scope != null) {
+			(this.unitBeingCompleted = parsedUnit).scope.checkParameterizedTypes();
+			if (buildFieldsAndMethods[i])
+				parsedUnit.scope.buildFieldsAndMethods();
+		}
+	}
+
+	this.unitBeingCompleted = null;
+}
 public MethodBinding computeArrayClone(MethodBinding objectClone) {
 	if (this.arrayClone == null) {
 		this.arrayClone = new MethodBinding(
@@ -637,6 +694,9 @@ public BinaryTypeBinding createBinaryTypeFrom(IBinaryType binaryType, PackageBin
 	}
 	packageBinding.addType(binaryBinding);
 	setAccessRestriction(binaryBinding, accessRestriction);
+	// need type annotations before processing methods (for @NonNullByDefault)
+	if (this.globalOptions.isAnnotationBasedNullAnalysisEnabled)
+		binaryBinding.scanTypeForNullDefaultAnnotation(binaryType, packageBinding, binaryBinding);
 	binaryBinding.cachePartsFrom(binaryType, needFieldsAndMethods);
 	return binaryBinding;
 }
@@ -777,6 +837,104 @@ public ParameterizedGenericMethodBinding createParameterizedGenericMethod(Method
 	cachedInfo[index] = parameterizedGenericMethod;
 	return parameterizedGenericMethod;
 }
+public PolymorphicMethodBinding createPolymorphicMethod(MethodBinding originalPolymorphicMethod, TypeBinding[] parameters) {
+	// cached info is array of already created polymorphic methods for this type
+	String key = new String(originalPolymorphicMethod.selector);
+	PolymorphicMethodBinding[] cachedInfo = (PolymorphicMethodBinding[]) this.uniquePolymorphicMethodBindings.get(key);
+	int parametersLength = parameters == null ? 0: parameters.length;
+	TypeBinding[] parametersTypeBinding = new TypeBinding[parametersLength]; 
+	for (int i = 0; i < parametersLength; i++) {
+		TypeBinding parameterTypeBinding = parameters[i];
+		if (parameterTypeBinding.id == TypeIds.T_null) {
+			parametersTypeBinding[i] = getType(JAVA_LANG_VOID);
+		} else {
+			parametersTypeBinding[i] = parameterTypeBinding.erasure();
+		}
+	}
+	boolean needToGrow = false;
+	int index = 0;
+	if (cachedInfo != null) {
+		nextCachedMethod :
+			// iterate existing polymorphic method for reusing one with same type arguments if any
+			for (int max = cachedInfo.length; index < max; index++) {
+				PolymorphicMethodBinding cachedMethod = cachedInfo[index];
+				if (cachedMethod == null) {
+					break nextCachedMethod;
+				}
+				if (cachedMethod.matches(parametersTypeBinding, originalPolymorphicMethod.returnType)) {
+					return cachedMethod;
+				}
+		}
+		needToGrow = true;
+	} else {
+		cachedInfo = new PolymorphicMethodBinding[5];
+		this.uniquePolymorphicMethodBindings.put(key, cachedInfo);
+	}
+	// grow cache ?
+	int length = cachedInfo.length;
+	if (needToGrow && index == length) {
+		System.arraycopy(cachedInfo, 0, cachedInfo = new PolymorphicMethodBinding[length*2], 0, length);
+		this.uniquePolymorphicMethodBindings.put(key, cachedInfo);
+	}
+	// add new binding
+	PolymorphicMethodBinding polymorphicMethod = new PolymorphicMethodBinding(
+			originalPolymorphicMethod,
+			parametersTypeBinding);
+	cachedInfo[index] = polymorphicMethod;
+	return polymorphicMethod;
+}
+public MethodBinding updatePolymorphicMethodReturnType(PolymorphicMethodBinding binding, TypeBinding typeBinding) {
+	// update the return type to be the given return type, but reuse existing binding if one can match
+	String key = new String(binding.selector);
+	PolymorphicMethodBinding[] cachedInfo = (PolymorphicMethodBinding[]) this.uniquePolymorphicMethodBindings.get(key);
+	boolean needToGrow = false;
+	int index = 0;
+	TypeBinding[] parameters = binding.parameters;
+	if (cachedInfo != null) {
+		nextCachedMethod :
+			// iterate existing polymorphic method for reusing one with same type arguments if any
+			for (int max = cachedInfo.length; index < max; index++) {
+				PolymorphicMethodBinding cachedMethod = cachedInfo[index];
+				if (cachedMethod == null) {
+					break nextCachedMethod;
+				}
+				if (cachedMethod.matches(parameters, typeBinding)) {
+					return cachedMethod;
+				}
+		}
+		needToGrow = true;
+	} else {
+		cachedInfo = new PolymorphicMethodBinding[5];
+		this.uniquePolymorphicMethodBindings.put(key, cachedInfo);
+	}
+	// grow cache ?
+	int length = cachedInfo.length;
+	if (needToGrow && index == length) {
+		System.arraycopy(cachedInfo, 0, cachedInfo = new PolymorphicMethodBinding[length*2], 0, length);
+		this.uniquePolymorphicMethodBindings.put(key, cachedInfo);
+	}
+	// add new binding
+	PolymorphicMethodBinding polymorphicMethod = new PolymorphicMethodBinding(
+			binding.original(),
+			typeBinding,
+			parameters);
+	cachedInfo[index] = polymorphicMethod;
+	return polymorphicMethod;
+}
+public ParameterizedMethodBinding createGetClassMethod(TypeBinding receiverType, MethodBinding originalMethod, Scope scope) {
+	// see if we have already cached this method for the given receiver type.
+	ParameterizedMethodBinding retVal = null;
+	if (this.uniqueGetClassMethodBinding == null) {
+		this.uniqueGetClassMethodBinding = new SimpleLookupTable(3);
+	} else {
+		retVal = (ParameterizedMethodBinding)this.uniqueGetClassMethodBinding.get(receiverType);
+	}
+	if (retVal == null) {
+		retVal = ParameterizedMethodBinding.instantiateGetClass(receiverType, originalMethod, scope);
+		this.uniqueGetClassMethodBinding.put(receiverType, retVal);
+	}
+	return retVal;
+}
 
 public ParameterizedTypeBinding createParameterizedType(ReferenceBinding genericType, TypeBinding[] typeArguments, ReferenceBinding enclosingType) {
 	// cached info is array of already created parameterized types for this type
@@ -914,8 +1072,6 @@ public AccessRestriction getAccessRestriction(TypeBinding type) {
  */
 public ReferenceBinding getCachedType(char[][] compoundName) {
 	if (compoundName.length == 1) {
-		if (this.defaultPackage == null)
-			return null;
 		return this.defaultPackage.getType0(compoundName[0]);
 	}
 	PackageBinding packageBinding = getPackage0(compoundName[0]);
@@ -928,6 +1084,18 @@ public ReferenceBinding getCachedType(char[][] compoundName) {
 	return packageBinding.getType0(compoundName[compoundName.length - 1]);
 }
 
+public char[][] getNullableAnnotationName() {
+	return this.globalOptions.nullableAnnotationName;
+}
+
+public char[][] getNonNullAnnotationName() {
+	return this.globalOptions.nonNullAnnotationName;
+}
+
+public char[][] getNonNullByDefaultAnnotationName() {
+	return this.globalOptions.nonNullByDefaultAnnotationName;
+}
+
 /* Answer the top level package named name if it exists in the cache.
 * Answer theNotFoundPackage if it could not be resolved the first time
 * it was looked up, otherwise answer null.
@@ -985,9 +1153,6 @@ public ReferenceBinding getType(char[][] compoundName) {
 	ReferenceBinding referenceBinding;
 
 	if (compoundName.length == 1) {
-		if (this.defaultPackage == null)
-			return null;
-
 		if ((referenceBinding = this.defaultPackage.getType0(compoundName[0])) == null) {
 			PackageBinding packageBinding = getPackage0(compoundName[0]);
 			if (packageBinding != null && packageBinding != TheNotFoundPackage)
@@ -1053,7 +1218,13 @@ private ReferenceBinding getTypeFromCompoundName(char[][] compoundName, boolean
 		packageBinding.addType(binding);
 	} else if (binding == TheNotFoundType) {
 		// report the missing class file first
-		this.problemReporter.isClassPathCorrect(compoundName, this.unitBeingCompleted, this.missingClassFileLocation);
+		if (!wasMissingType) {
+			/* Since missing types have been already been complained against while producing binaries, there is no class path 
+			 * misconfiguration now that did not also exist in some equivalent form while producing the class files which encode 
+			 * these missing types. So no need to bark again. Note that wasMissingType == true signals a type referenced in a .class 
+			 * file which could not be found when the binary was produced. See https://bugs.eclipse.org/bugs/show_bug.cgi?id=364450 */
+			this.problemReporter.isClassPathCorrect(compoundName, this.unitBeingCompleted, this.missingClassFileLocation);
+		}
 		// create a proxy for the missing BinaryType
 		binding = createMissingType(null, compoundName);
 	} else if (!isParameterized) {
@@ -1269,9 +1440,12 @@ public MethodVerifier methodVerifier() {
 }
 
 public MethodVerifier newMethodVerifier() {
-	return this.globalOptions.sourceLevel < ClassFileConstants.JDK1_5
-		? new MethodVerifier(this)
-		: new MethodVerifier15(this); // covariance only if sourceLevel is >= 1.5
+	/* Always use MethodVerifier15. Even in a 1.4 project, we must internalize type variables and
+	   observe any parameterization of super class and/or super interfaces in order to be able to
+	   detect overriding in the presence of generics.
+	   See https://bugs.eclipse.org/bugs/show_bug.cgi?id=324850
+	 */
+	return new MethodVerifier15(this);
 }
 
 public void releaseClassFiles(org.eclipse.jdt.internal.compiler.ClassFile[] classFiles) {
@@ -1297,7 +1471,10 @@ public void reset() {
 	this.uniqueRawTypeBindings = new SimpleLookupTable(3);
 	this.uniqueWildcardBindings = new SimpleLookupTable(3);
 	this.uniqueParameterizedGenericMethodBindings = new SimpleLookupTable(3);
+	this.uniquePolymorphicMethodBindings = new SimpleLookupTable(3);
+	this.uniqueGetClassMethodBinding = null;
 	this.missingTypes = null;
+	this.typesBeingConnected = new HashSet();
 
 	for (int i = this.units.length; --i >= 0;)
 		this.units[i] = null;
@@ -1306,6 +1483,7 @@ public void reset() {
 	this.unitBeingCompleted = null; // in case AbortException occurred
 
 	this.classFilePool.reset();
+
 	// name environment has a longer life cycle, and must be reset in
 	// the code which created it.
 }
@@ -1351,4 +1529,17 @@ void updateCaches(UnresolvedReferenceBinding unresolvedType, ReferenceBinding re
 		}
 	}
 }
+
+public IQualifiedTypeResolutionListener[] resolutionListeners = new IQualifiedTypeResolutionListener[0];
+
+public void addResolutionListener(IQualifiedTypeResolutionListener resolutionListener) {
+	int length = this.resolutionListeners.length;
+	for (int i = 0; i < length; i++){
+		if (this.resolutionListeners[i].equals(resolutionListener))
+			return;
+	}
+	System.arraycopy(this.resolutionListeners, 0,
+			this.resolutionListeners = new IQualifiedTypeResolutionListener[length + 1], 0, length);
+	this.resolutionListeners[length] = resolutionListener;
+}
 }
