@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,13 @@
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contributions for
+ *								bug 186342 - [compiler][null] Using annotations for null checking
+ *								bug 367203 - [compiler][null] detect assigning null to nonnull argument
+ *								bug 365519 - editorial cleanup after bug 186342 and bug 365387
+ *								bug 365662 - [compiler][null] warn on contradictory and redundant null annotations
+ *								bug 365531 - [compiler][null] investigate alternative strategy for internally encoding nullness defaults
+ *								bug 388281 - [compiler][null] inheritance of null annotations as an option
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.lookup;
 
@@ -34,6 +41,9 @@ public class MethodBinding extends Binding {
 	char[] signature;
 	public long tagBits;
 
+	/** Store nullness information from annotation (incl. applicable default). */
+	public Boolean[] parameterNonNullness;  // TRUE means @NonNull declared, FALSE means @Nullable declared, null means nothing declared
+
 protected MethodBinding() {
 	// for creating problem or synthetic method
 }
@@ -161,13 +171,22 @@ MethodBinding asRawMethod(LookupEnvironment env) {
 			arguments[i] = env.convertToRawType(var.upperBound(), false /*do not force conversion of enclosing types*/);
 		} else {
 			// use an intersection type to retain full bound information if more than 1 bound
-			TypeBinding rawSuperclass = env.convertToRawType(var.superclass(), false);
 			TypeBinding[] itsSuperinterfaces = var.superInterfaces();
 			int superLength = itsSuperinterfaces.length;
-			TypeBinding[] rawSuperinterfaces = new TypeBinding[superLength];
-			for (int s = 0; s < superLength; s++)
-				rawSuperinterfaces[s] = env.convertToRawType(itsSuperinterfaces[s], false);
-			arguments[i] = env.createWildcard(null, 0, rawSuperclass, rawSuperinterfaces, org.eclipse.jdt.internal.compiler.ast.Wildcard.EXTENDS);
+			TypeBinding rawFirstBound = null;
+			TypeBinding[] rawOtherBounds = null;
+			if (var.boundsCount() == superLength) {
+				rawFirstBound = env.convertToRawType(itsSuperinterfaces[0], false);
+				rawOtherBounds = new TypeBinding[superLength - 1];
+				for (int s = 1; s < superLength; s++)
+					rawOtherBounds[s - 1] = env.convertToRawType(itsSuperinterfaces[s], false);
+			} else {
+				rawFirstBound = env.convertToRawType(var.superclass(), false);
+				rawOtherBounds = new TypeBinding[superLength];
+				for (int s = 0; s < superLength; s++)
+					rawOtherBounds[s] = env.convertToRawType(itsSuperinterfaces[s], false);
+			}
+			arguments[i] = env.createWildcard(null, 0, rawFirstBound, rawOtherBounds, org.eclipse.jdt.internal.compiler.ast.Wildcard.EXTENDS);
 		}
 	}
 	return env.createParameterizedGenericMethod(this, arguments);
@@ -278,8 +297,8 @@ public final boolean canBeSeenBy(TypeBinding receiverType, InvocationSite invoca
 		// AND the invocationType and the declaringClass have a common enclosingType
 		receiverCheck: {
 			if (receiverType != this.declaringClass) {
-				// special tolerance for type variable direct bounds
-				if (receiverType.isTypeVariable() && ((TypeVariableBinding) receiverType).isErasureBoundTo(this.declaringClass.erasure()))
+				// special tolerance for type variable direct bounds, but only if compliance <= 1.6, see: https://bugs.eclipse.org/bugs/show_bug.cgi?id=334622
+				if (scope.compilerOptions().complianceLevel <= ClassFileConstants.JDK1_6 && receiverType.isTypeVariable() && ((TypeVariableBinding) receiverType).isErasureBoundTo(this.declaringClass.erasure()))
 					break receiverCheck;
 				return false;
 			}
@@ -312,9 +331,13 @@ public final boolean canBeSeenBy(TypeBinding receiverType, InvocationSite invoca
 	if (receiverType instanceof ArrayBinding)
 		return false;
 	TypeBinding originalDeclaringClass = this.declaringClass.original();
-	ReferenceBinding currentType = (ReferenceBinding) receiverType;
+	ReferenceBinding currentType = (ReferenceBinding) (receiverType);
 	do {
-		if (originalDeclaringClass == currentType.original()) return true;
+		if (currentType.isCapture()) { // https://bugs.eclipse.org/bugs/show_bug.cgi?id=285002
+			if (originalDeclaringClass == currentType.erasure().original()) return true;
+		} else {
+			if (originalDeclaringClass == currentType.original()) return true;
+		}
 		PackageBinding currentPackage = currentType.fPackage;
 		// package could be null for wildcards/intersection types, ignore and recurse in superclass
 		if (currentPackage != null && currentPackage != declaringPackage) return false;
@@ -426,6 +449,54 @@ public final char[] constantPoolName() {
 	return this.selector;
 }
 
+/**
+ * After method verifier has finished, fill in missing @NonNull specification from the applicable default.
+ */
+protected void fillInDefaultNonNullness(AbstractMethodDeclaration sourceMethod) {
+	if (this.parameterNonNullness == null)
+		this.parameterNonNullness = new Boolean[this.parameters.length];
+	boolean added = false;
+	int length = this.parameterNonNullness.length;
+	for (int i = 0; i < length; i++) {
+		if (this.parameters[i].isBaseType())
+			continue;
+		if (this.parameterNonNullness[i] == null) {
+			added = true;
+			this.parameterNonNullness[i] = Boolean.TRUE;
+			if (sourceMethod != null) {
+				sourceMethod.arguments[i].binding.tagBits |= TagBits.AnnotationNonNull;
+			}
+		} else if (sourceMethod != null && this.parameterNonNullness[i].booleanValue()) {
+			sourceMethod.scope.problemReporter().nullAnnotationIsRedundant(sourceMethod, i);
+		}
+	}
+	if (added)
+		this.tagBits |= TagBits.HasParameterAnnotations;
+	if (   this.returnType != null
+		&& !this.returnType.isBaseType()
+		&& (this.tagBits & (TagBits.AnnotationNonNull|TagBits.AnnotationNullable)) == 0)
+	{
+		this.tagBits |= TagBits.AnnotationNonNull;
+	} else if (sourceMethod != null && (this.tagBits & TagBits.AnnotationNonNull) != 0) {
+		sourceMethod.scope.problemReporter().nullAnnotationIsRedundant(sourceMethod, -1/*signifies method return*/);
+	}
+}
+
+public MethodBinding findOriginalInheritedMethod(MethodBinding inheritedMethod) {
+	MethodBinding inheritedOriginal = inheritedMethod.original();
+	TypeBinding superType = this.declaringClass.findSuperTypeOriginatingFrom(inheritedOriginal.declaringClass);
+	if (superType == null || !(superType instanceof ReferenceBinding)) return null;
+
+	if (inheritedOriginal.declaringClass != superType) {
+		// must find inherited method with the same substituted variables
+		MethodBinding[] superMethods = ((ReferenceBinding) superType).getMethods(inheritedOriginal.selector, inheritedOriginal.parameters.length);
+		for (int m = 0, l = superMethods.length; m < l; m++)
+			if (superMethods[m].original() == inheritedOriginal)
+				return superMethods[m];
+	}
+	return inheritedOriginal;
+}
+
 /**
  * <pre>
  *<typeParam1 ... typeParamM>(param1 ... paramN)returnType thrownException1 ... thrownExceptionP
@@ -496,6 +567,13 @@ public long getAnnotationTagBits() {
 			AbstractMethodDeclaration methodDecl = typeDecl.declarationOf(originalMethod);
 			if (methodDecl != null)
 				ASTNode.resolveAnnotations(methodDecl.scope, methodDecl.annotations, originalMethod);
+			long nullDefaultBits = this.tagBits & (TagBits.AnnotationNonNullByDefault|TagBits.AnnotationNullUnspecifiedByDefault);
+			if (nullDefaultBits != 0 && this.declaringClass instanceof SourceTypeBinding) {
+				SourceTypeBinding declaringSourceType = (SourceTypeBinding) this.declaringClass;
+				if (declaringSourceType.checkRedundantNullnessDefaultOne(methodDecl, methodDecl.annotations, nullDefaultBits)) {
+					declaringSourceType.checkRedundantNullnessDefaultRecurse(methodDecl, methodDecl.annotations, nullDefaultBits);
+				}
+			}
 		}
 	}
 	return originalMethod.tagBits;
@@ -524,86 +602,6 @@ public Object getDefaultValue() {
 	return holder == null ? null : holder.getDefaultValue();
 }
 
-/**
- * Return the highest method/constructor in supertype hierarchy with same selector and arguments
- */
-public MethodBinding getHighestOverridenMethod(LookupEnvironment environment) {
-	MethodBinding bestMethod = this;
-    ReferenceBinding currentType = this.declaringClass;
-    if (this.isConstructor()) {
-    	// walk superclasses - only
-    	do {
-    		MethodBinding superMethod = currentType.getExactConstructor(this.parameters);
-    		if (superMethod != null) {
-    			bestMethod = superMethod;
-    		}
-    	} while ((currentType = currentType.superclass()) != null);
-    	return bestMethod;
-    }
-    MethodVerifier verifier = environment.methodVerifier();
-	// walk superclasses
-	ReferenceBinding[] interfacesToVisit = null;
-	int nextPosition = 0;
-	do {
-		MethodBinding[] superMethods = currentType.getMethods(this.selector);
-		for (int i = 0, length = superMethods.length; i < length; i++) {
-			if (verifier.doesMethodOverride(this, superMethods[i])) {
-				bestMethod = superMethods[i];
-				break;
-			}
-		}
-		ReferenceBinding[] itsInterfaces = currentType.superInterfaces();
-		if (itsInterfaces != null && itsInterfaces != Binding.NO_SUPERINTERFACES) {
-			if (interfacesToVisit == null) {
-				interfacesToVisit = itsInterfaces;
-				nextPosition = interfacesToVisit.length;
-			} else {
-				int itsLength = itsInterfaces.length;
-				if (nextPosition + itsLength >= interfacesToVisit.length)
-					System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
-				nextInterface : for (int a = 0; a < itsLength; a++) {
-					ReferenceBinding next = itsInterfaces[a];
-					for (int b = 0; b < nextPosition; b++)
-						if (next == interfacesToVisit[b]) continue nextInterface;
-					interfacesToVisit[nextPosition++] = next;
-				}
-			}
-		}
-	} while ((currentType = currentType.superclass()) != null);
-	if (bestMethod.declaringClass.id == TypeIds.T_JavaLangObject) {
-		return bestMethod;
-	}
-	// walk superinterfaces
-	for (int i = 0; i < nextPosition; i++) {
-		currentType = interfacesToVisit[i];
-		MethodBinding[] superMethods = currentType.getMethods(this.selector);
-		for (int j = 0, length = superMethods.length; j < length; j++) {
-			MethodBinding superMethod = superMethods[j];
-			if (verifier.doesMethodOverride(this, superMethod)) {
-				TypeBinding bestReturnType = bestMethod.returnType;
-				if (bestReturnType == superMethod.returnType
-						|| bestMethod.returnType.findSuperTypeOriginatingFrom(superMethod.returnType) != null) {
-					bestMethod = superMethod;
-				}
-				break;
-			}
-		}
-		ReferenceBinding[] itsInterfaces = currentType.superInterfaces();
-		if (itsInterfaces != null && itsInterfaces != Binding.NO_SUPERINTERFACES) {
-			int itsLength = itsInterfaces.length;
-			if (nextPosition + itsLength >= interfacesToVisit.length)
-				System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
-			nextInterface : for (int a = 0; a < itsLength; a++) {
-				ReferenceBinding next = itsInterfaces[a];
-				for (int b = 0; b < nextPosition; b++)
-					if (next == interfacesToVisit[b]) continue nextInterface;
-				interfacesToVisit[nextPosition++] = next;
-			}
-		}
-	}	
-	return bestMethod;
-}
-
 /**
  * @return the annotations for each of the method parameters or <code>null></code>
  * 	if there's no parameter or no annotation at all.
@@ -804,10 +802,12 @@ public final boolean isUsed() {
 
 /* Answer true if the receiver method has varargs
 */
-public final boolean isVarargs() {
+public boolean isVarargs() {
 	return (this.modifiers & ClassFileConstants.AccVarargs) != 0;
 }
-
+public boolean isPolymorphic() {
+	return false;
+}
 /* Answer true if the receiver's declaring type is deprecated (or any of its enclosing types)
 */
 public final boolean isViewedAsDeprecated() {
@@ -821,7 +821,7 @@ public final int kind() {
 */
 
 /**
- * Returns the original method (as opposed to parameterized instances)
+ * Returns the original method (as opposed to parameterized/polymorphic instances)
  */
 public MethodBinding original() {
 	return this;
@@ -1013,7 +1013,7 @@ public final char[] signature(ClassFile classFile) {
 		return this.signature;
 	}
 
-	StringBuffer buffer = new StringBuffer(this.parameters.length + 1 * 20);
+	StringBuffer buffer = new StringBuffer((this.parameters.length + 1) * 20);
 	buffer.append('(');
 
 	TypeBinding[] targetParameters = this.parameters;
@@ -1095,6 +1095,9 @@ public final int sourceEnd() {
 	return method.sourceEnd;
 }
 public AbstractMethodDeclaration sourceMethod() {
+	if (isSynthetic()) {
+		return null;
+	}
 	SourceTypeBinding sourceType;
 	try {
 		sourceType = (SourceTypeBinding) this.declaringClass;
@@ -1103,9 +1106,11 @@ public AbstractMethodDeclaration sourceMethod() {
 	}
 
 	AbstractMethodDeclaration[] methods = sourceType.scope.referenceContext.methods;
-	for (int i = methods.length; --i >= 0;)
-		if (this == methods[i].binding)
-			return methods[i];
+	if (methods != null) {
+		for (int i = methods.length; --i >= 0;)
+			if (this == methods[i].binding)
+				return methods[i];
+	}
 	return null;
 }
 public final int sourceStart() {
@@ -1165,4 +1170,11 @@ public String toString() {
 public TypeVariableBinding[] typeVariables() {
 	return this.typeVariables;
 }
+public boolean hasNonNullDefault() {
+	if ((this.tagBits & TagBits.AnnotationNonNullByDefault) != 0)
+		return true;
+	if ((this.tagBits & TagBits.AnnotationNullUnspecifiedByDefault) != 0)
+		return false;
+	return this.declaringClass.hasNonNullDefault();
+}
 }
